cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c GET_PDF(inlo,x,mu,pdf)                                               c
c        interface calling the parton densities                        c
c                                                                      c
c   input  inlo=0,1 for LO, NLO set, dummy parameter for CTEQ calls    c
c          x        momentum fraction (cut for safety)                 c
c          mu       factorization scale                                c
c                                                                      c
c   output pdf(-6:6) just like CteQ: g,u,d,s,c,b,(t)                   c
c                                                                      c
c n.b. lambda_qcd has to be set accordingly in GET_LAMBDA_QCD          c
c      cteq arrays have to initialized using INIT_PDF                  c
c                                                                      c
c If for some reason you do not like Cteq5, you can switch to Cteq6:   c
c   (1) change the actual pdf call below                               c
c   (2) change the lambda_QCD values below                             c
c   (3) change the initialization call below                           c
c   note for (1)-(3) the Cteq6 values are commented out using `ctq6'   c 
c   (4) link the proper routine in the Makefile                        c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine GET_PDF(inlo,x,mu,pdf)

      implicit none
      
      integer inlo,i1,iset
      real*8  x,mu,xphoton,pdf(-6:6),pdf_mstw(-6:6)
      real*8  Ctq6Pdf,Ctq5Pdf,CT10Pdf
      character*50 path_mstw

      integer ipdf_common
      common/PARAS_PDF/ipdf_common

      logical ldebug
      parameter( ldebug=.false. )

      if (ldebug) print*, " GET_PDF: initialized ",ipdf_common
      if (ldebug) print*, " GET_PDF: input parameters ",x,mu

c              initialize the array 
      do i1=-6,6,1
         pdf(i1) = 0.D0
      end do

c              check if range makes sense, otherwise return zero
      if ( (x.ge.1.D0) .or. (x.lt.1.D-5) ) then
         print*, " GET_PDF: problem with x ",x
         return
      end if

c              if scale too low, return zero
      if (mu.lt.5.D0) then 
         print*, " GET_PDF: problem with too small muF ",mu
         return
      end if

c              if scale too high, set to maximum
      if (mu.gt.1.5D3) then
         print*, " GET_PDF: problem with too large muF ",mu
         mu = 1.5D3
      end if
         
c              call MSTW first, no initialization instead
      if ((ipdf_common.ge.100).and.(ipdf_common.lt.200)) then 
         if (ipdf_common.eq.100) then
            path_mstw = "./Pro2_pdf/mstw2008nlo"
         else if ((ipdf_common.ge.101).and.(ipdf_common.le.140)) then
            path_mstw = "./Pro2_pdf/mstw2008nlo.68cl"
         end if 
         if (inlo.eq.1) then
            iset = ipdf_common - 100
            call GetAllPDFsAlt(path_mstw,iset,x,mu,pdf_mstw,xphoton)
         end if 
      end if

      do i1=-5,5,1
         if (inlo.eq.0) then
            pdf(i1) = Ctq6Pdf(i1,x,mu)
         else if (inlo.eq.1) then 
            if (ipdf_common.eq.-1) then 
               pdf(i1) = Ctq6Pdf(i1,x,mu)
            else if ((ipdf_common.ge.0).and.(ipdf_common.lt.100)) then 
               pdf(i1) = CT10Pdf(i1,x,mu)
            else if ((ipdf_common.ge.100).and.(ipdf_common.lt.200)) then 
               if (i1.eq.1) then 
                  pdf(i1) = pdf_mstw(2)
               else if (i1.eq.2) then 
                  pdf(i1) = pdf_mstw(1)
               else if (i1.eq.-1) then 
                  pdf(i1) = pdf_mstw(-2)
               else if (i1.eq.-2) then 
                  pdf(i1) = pdf_mstw(-1)
               else
                  pdf(i1) = pdf_mstw(i1)
               end if
               pdf(i1) = pdf(i1)/x
            end if
         end if
         if (ldebug) print*, " GET_PDF: pdf call ",i1,pdf(i1)
      end do

ctq5         pdf(i1) = Ctq5Pdf(i1,x,mu)
   
      return
      end

c -----------------------------------------------
c   this initialization is needed for a consistent alpha_s
      subroutine GET_LAMBDA_QCD(inlo,lambda_qcd)
      
      implicit none

      integer inlo
      real*8  lambda_qcd

      integer ipdf_common
      common/PARAS_PDF/ipdf_common

c alphas(mZ) = 0.11707 means lQCD5 = 0.251 (tested with example code)

      if (inlo.eq.0) then 
         lambda_qcd = 0.165D0
      else if (inlo.eq.1) then 
         if (ipdf_common.lt.100) then
            lambda_qcd = 0.226D0
         else if ((ipdf_common.ge.100).and.(ipdf_common.lt.200)) then 
            lambda_qcd = 0.215D0
         else 
            print*, " GET_LAMBA_QCD: pdf wrong, fingers crossed "
         end if
      else
         print*, " GET_LAMBDA_QCD: inlo set wrongly ",inlo
      end if

ctq5         lambda_qcd = 0.146D0
         
      return
      end 

c -----------------------------------------------
c   this initialization function is needed for all pdf routines
c    ipdf = -1     Cteq6 (always used for LO)
c    ipdf =  0-52  CT10 sets, 0 the central set
      subroutine INIT_PDF(inlo,npdf)

      implicit none

      integer inlo,npdf,icount_ct10

      integer ipdf_common
      common/PARAS_PDF/ipdf_common

      ipdf_common = npdf

      if (inlo.eq.0) then 
         call SetCtq6(4)
         print*, " INIT_PDF: LO CTQ6 parton set "
      else if (inlo.eq.1) then 
         if (ipdf_common.eq.-1) then 
            call SetCtq6(400)
            print*, " INIT_PDF: NLO CTQ6 parton set "
         else if ((ipdf_common.ge.0).and.(ipdf_common.lt.100)) then 
            icount_ct10 = ipdf_common + 100
            call SetCT10(icount_ct10)
            print*, " INIT_PDF: NLO CT10 parton set ",icount_ct10
         else if ((ipdf_common.ge.100).and.(ipdf_common.lt.200)) then 
            print*, " INIT_PDF: NLO MSTW parton set "
         else 
            print*, " INIT_PDF: pdf wrong, fingers crossed ",npdf
         end if
      else
         print*, " INIT_PDF: inlo set wrongly ",inlo
      end if

ctp      print*, " INIT_PDF: initialized ",inlo,ipdf_common

ctq5         call SetCtq5(3)
ctq6         call SetCtq6(1)
ctq5         call SetCtq5(8)
         
      return
      end 


