cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     THE SCALING FUNCTIONS                                            c
c                                                                      c
c     LE_QG(MASSIN,LUMI)                                                c
c                                                                      c
c     INPUT :                                                          c
c                                                                      c
c       MASSIN(1)  = s                                                 c
c       MASSIN(2)  = t2                                                c
c       MASSIN(3)  = s4                                                c
c       MASSIN(6)  = m1                                                c
c       MASSIN(12) = qr                                                c
c       MASSIN(13) = qf                                                c
c       MASSIN(25) = gamma_s                                           c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      real*8 function LE_QG(massin,lumi)

      implicit none 

      real*8     massin(1:30),lumi(1:3),pi,Nc,alphas
     &          ,Ar2,st,log_qf,log_all,MM_qg
     &          ,s,t2,s4,m1,t2s,u2,u2s,m12
     &          ,hardfac
     &          ,ANG_fin(0:9,0:9,-2:2,-2:2)

      pi = 4.D0*atan(1.D0)
      Nc = 3.D0

      s    = massin(1)
      t2   = massin(2)
      s4   = massin(3)
      m1   = massin(6)
      m12  = m1**2 

c               the factorization scale 
      log_qf = log( massin(13)**2/m1**2 ) 
      log_all = log_qf - log(s4/m1**2) + log(1.D0+m1**2/s4)

c               real kinematics built in
      t2s = t2 - m1**2 

      u2  = s4 - s - t2 + m1**2 
      u2s = u2 - m1**2 
ctp      u2s = u1
ctp      u2  = u1 + m1**2

c               gs**2=4*pi*alpha_s is cut, re-appears as nlo later 
      Ar2     = 1.D0
      st      = 1.D0
      alphas  = 1.D0/4.D0/pi
      hardfac = 1.D0 

c               the angular functions 
      call ANGULAR_ARRAY_LE_QG(massin,ANG_fin)

c               the form output
c               replacements: [] -> ()
      MM_qg =
     +  + Ar2*Nc**(-1)*st**2*Pi**3*alphas**2*hardfac*log_all * ( 128*
     +    s**(-1)*(s+u2)**(-1)*m1**2*t2*u2*s4**(-2) + 128*s**(-1)*
     +    (s+u2)**(-1)*t2*u2*s4**(-1) + 128*s**(-1)*m1**2*t2*u2**2*
     +    s4**(-2)*t2s**(-2) - 64*s**(-1)*m1**2*t2*s4**(-2) - 64*
     +    s**(-1)*m1**2*u2**2*s4**(-2)*t2s**(-1) - 192*s**(-1)*m1**2*
     +    u2**2*s4**(-1)*t2s**(-2) - 128*s**(-1)*m1**2*s4**(-1) - 192*
     +    s**(-1)*m1**4*u2**2*s4**(-2)*t2s**(-2) + 128*s**(-1)*m1**4*
     +    u2**2*s4**(-1)*t2s**(-3) - 128*s**(-1)*m1**4*s4**(-2) + 128*
     +    s**(-1)*m1**4*s4**(-1)*t2s**(-1) - 128*s**(-1)*m1**6*t2**(-1)
     +    *u2*t2s**(-3) - 128*s**(-1)*m1**6*t2**(-1)*u2s**(-1)*
     +    t2s**(-1) + 128*s**(-1)*m1**6*u2*s4**(-1)*t2s**(-3) + 128*
     +    s**(-1)*m1**6*u2**2*s4**(-2)*t2s**(-3) + 128*s**(-1)*m1**6*
     +    s4**(-2)*t2s**(-1) + 128*s**(-1)*m1**6*s4**(-1)*u2s**(-1)*
     +    t2s**(-1) - 128*s**(-1)*m1**8*t2**(-1)*u2*s4**(-1)*t2s**(-3)
     +     - 128*s**(-1)*m1**8*t2**(-1)*s4**(-1)*u2s**(-1)*t2s**(-1) - 
     +    128*s**(-1)*m1**8*t2**(-1)*t2s**(-3) )
      MM_qg = MM_qg + Ar2*Nc**(-1)*st**2*Pi**3*alphas**2*hardfac*
     + log_all * ( 128*s**(-1)*m1**8*u2*s4**(-2)*t2s**(-3) + 128*
     +    s**(-1)*m1**8*s4**(-2)*u2s**(-1)*t2s**(-1) + 128*s**(-1)*
     +    m1**8*s4**(-1)*t2s**(-3) - 128*s**(-1)*m1**10*t2**(-1)*
     +    s4**(-1)*t2s**(-3) - 128*s**(-1)*m1**10*t2**(-1)*u2s**(-1)*
     +    t2s**(-3) + 128*s**(-1)*m1**10*s4**(-2)*t2s**(-3) + 128*
     +    s**(-1)*m1**10*s4**(-1)*u2s**(-1)*t2s**(-3) - 128*s**(-1)*
     +    m1**12*t2**(-1)*s4**(-1)*u2s**(-1)*t2s**(-3) + 128*s**(-1)*
     +    m1**12*s4**(-2)*u2s**(-1)*t2s**(-3) + 128*s**(-1)*t2*u2**2*
     +    s4**(-1)*t2s**(-2) - 64*s**(-1)*t2*s4**(-1) - 64*s**(-1)*
     +    u2**2*s4**(-1)*t2s**(-1) + 128*s*(t2*u2+s*m12)**(-2)*m1**4 - 
     +    384*s*(t2*u2+s*m12)**(-2)*m1**6*u2*t2s**(-2) + 128*s*
     +    (t2*u2+s*m12)**(-2)*m1**6*s4**(-1) + 256*s*
     +    (t2*u2+s*m12)**(-2)*m1**6*t2s**(-1) - 384*s*
     +    (t2*u2+s*m12)**(-2)*m1**8*u2*s4**(-1)*t2s**(-2) + 256*s*
     +    (t2*u2+s*m12)**(-2)*m1**8*s4**(-1)*t2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc**(-1)*st**2*Pi**3*alphas**2*hardfac*
     + log_all * ( 256*s*(t2*u2+s*m12)**(-2)*m1**8*u2s**(-1)*t2s**(-1)
     +     - 384*s*(t2*u2+s*m12)**(-2)*m1**8*t2s**(-2) + 256*s*
     +    (t2*u2+s*m12)**(-2)*m1**10*s4**(-1)*u2s**(-1)*t2s**(-1) - 384
     +    *s*(t2*u2+s*m12)**(-2)*m1**10*s4**(-1)*t2s**(-2) - 384*s*
     +    (t2*u2+s*m12)**(-2)*m1**10*u2s**(-1)*t2s**(-2) - 384*s*
     +    (t2*u2+s*m12)**(-2)*m1**12*s4**(-1)*u2s**(-1)*t2s**(-2) - 256
     +    *s*(t2*u2+s*m12)**(-1)*m1**4*u2s**(-1)*t2s**(-1) + 512*s*
     +    (t2*u2+s*m12)**(-1)*m1**4*t2s**(-2) - 256*s*
     +    (t2*u2+s*m12)**(-1)*m1**6*s4**(-1)*u2s**(-1)*t2s**(-1) + 512*
     +    s*(t2*u2+s*m12)**(-1)*m1**6*s4**(-1)*t2s**(-2) + 768*s*
     +    (t2*u2+s*m12)**(-1)*m1**6*u2s**(-1)*t2s**(-2) - 384*s*
     +    (t2*u2+s*m12)**(-1)*m1**6*t2s**(-3) + 768*s*
     +    (t2*u2+s*m12)**(-1)*m1**8*s4**(-1)*u2s**(-1)*t2s**(-2) - 384*
     +    s*(t2*u2+s*m12)**(-1)*m1**8*s4**(-1)*t2s**(-3) + 256*s*
     +    (t2*u2+s*m12)**(-1)*m1**8*u2s**(-2)*t2s**(-2) )
      MM_qg = MM_qg + Ar2*Nc**(-1)*st**2*Pi**3*alphas**2*hardfac*
     + log_all * (  - 768*s*(t2*u2+s*m12)**(-1)*m1**8*u2s**(-1)*
     +    t2s**(-3) + 256*s*(t2*u2+s*m12)**(-1)*m1**10*s4**(-1)*
     +    u2s**(-2)*t2s**(-2) - 768*s*(t2*u2+s*m12)**(-1)*m1**10*
     +    s4**(-1)*u2s**(-1)*t2s**(-3) - 384*s*(t2*u2+s*m12)**(-1)*
     +    m1**10*u2s**(-2)*t2s**(-3) - 384*s*(t2*u2+s*m12)**(-1)*m1**12
     +    *s4**(-1)*u2s**(-2)*t2s**(-3) + 64*s*m1**2*s4**(-2)*t2s**(-1)
     +     + 256*s*m1**2*s4**(-1)*u2s**(-1)*t2s**(-1) - 320*s*m1**2*
     +    s4**(-1)*t2s**(-2) + 256*s*m1**4*s4**(-2)*u2s**(-1)*t2s**(-1)
     +     - 320*s*m1**4*s4**(-2)*t2s**(-2) - 768*s*m1**4*s4**(-1)*
     +    u2s**(-1)*t2s**(-2) + 384*s*m1**4*s4**(-1)*t2s**(-3) - 768*s*
     +    m1**6*s4**(-2)*u2s**(-1)*t2s**(-2) + 384*s*m1**6*s4**(-2)*
     +    t2s**(-3) - 256*s*m1**6*s4**(-1)*u2s**(-2)*t2s**(-2) + 768*s*
     +    m1**6*s4**(-1)*u2s**(-1)*t2s**(-3) - 256*s*m1**8*s4**(-2)*
     +    u2s**(-2)*t2s**(-2) + 768*s*m1**8*s4**(-2)*u2s**(-1)*
     +    t2s**(-3) )
      MM_qg = MM_qg + Ar2*Nc**(-1)*st**2*Pi**3*alphas**2*hardfac*
     + log_all * ( 384*s*m1**8*s4**(-1)*u2s**(-2)*t2s**(-3) + 384*s*
     +    m1**10*s4**(-2)*u2s**(-2)*t2s**(-3) + 64*s*s4**(-1)*t2s**(-1)
     +     + 128*s**2*(t2*u2+s*m12)**(-2)*m1**6*u2s**(-1)*t2s**(-1) - 
     +    384*s**2*(t2*u2+s*m12)**(-2)*m1**6*t2s**(-2) + 128*s**2*
     +    (t2*u2+s*m12)**(-2)*m1**8*s4**(-1)*u2s**(-1)*t2s**(-1) - 384*
     +    s**2*(t2*u2+s*m12)**(-2)*m1**8*s4**(-1)*t2s**(-2) - 384*s**2*
     +    (t2*u2+s*m12)**(-2)*m1**8*u2s**(-1)*t2s**(-2) - 384*s**2*
     +    (t2*u2+s*m12)**(-2)*m1**10*s4**(-1)*u2s**(-1)*t2s**(-2) + 128
     +    *s**2*(t2*u2+s*m12)**(-1)*m1**4*u2s**(-1)*t2s**(-2) + 128*
     +    s**2*(t2*u2+s*m12)**(-1)*m1**6*s4**(-1)*u2s**(-1)*t2s**(-2)
     +     + 128*s**2*(t2*u2+s*m12)**(-1)*m1**6*u2s**(-2)*t2s**(-2) - 
     +    256*s**2*(t2*u2+s*m12)**(-1)*m1**6*u2s**(-1)*t2s**(-3) + 128*
     +    s**2*(t2*u2+s*m12)**(-1)*m1**8*s4**(-1)*u2s**(-2)*t2s**(-2)
     +     - 256*s**2*(t2*u2+s*m12)**(-1)*m1**8*s4**(-1)*u2s**(-1)*
     +    t2s**(-3) )
      MM_qg = MM_qg + Ar2*Nc**(-1)*st**2*Pi**3*alphas**2*hardfac*
     + log_all * (  - 384*s**2*(t2*u2+s*m12)**(-1)*m1**8*u2s**(-2)*
     +    t2s**(-3) - 384*s**2*(t2*u2+s*m12)**(-1)*m1**10*s4**(-1)*
     +    u2s**(-2)*t2s**(-3) - 128*s**2*m1**2*s4**(-1)*u2s**(-1)*
     +    t2s**(-2) - 128*s**2*m1**4*s4**(-2)*u2s**(-1)*t2s**(-2) - 128
     +    *s**2*m1**4*s4**(-1)*u2s**(-2)*t2s**(-2) + 256*s**2*m1**4*
     +    s4**(-1)*u2s**(-1)*t2s**(-3) - 128*s**2*m1**6*s4**(-2)*
     +    u2s**(-2)*t2s**(-2) + 256*s**2*m1**6*s4**(-2)*u2s**(-1)*
     +    t2s**(-3) + 384*s**2*m1**6*s4**(-1)*u2s**(-2)*t2s**(-3) + 384
     +    *s**2*m1**8*s4**(-2)*u2s**(-2)*t2s**(-3) - 128*s**3*
     +    (t2*u2+s*m12)**(-2)*m1**6*u2s**(-1)*t2s**(-2) - 128*s**3*
     +    (t2*u2+s*m12)**(-2)*m1**8*s4**(-1)*u2s**(-1)*t2s**(-2) - 128*
     +    s**3*(t2*u2+s*m12)**(-1)*m1**6*u2s**(-2)*t2s**(-3) - 128*s**3
     +    *(t2*u2+s*m12)**(-1)*m1**8*s4**(-1)*u2s**(-2)*t2s**(-3) + 128
     +    *s**3*m1**4*s4**(-1)*u2s**(-2)*t2s**(-3) + 128*s**3*m1**6*
     +    s4**(-2)*u2s**(-2)*t2s**(-3) )
      MM_qg = MM_qg + Ar2*Nc**(-1)*st**2*Pi**3*alphas**2*hardfac*
     + log_all * ( 256*(s+u2)**(-1)*m1**2*s4**(-1) + 128*(s+u2)**(-1)*
     +    m1**4*s4**(-2) + 128*(s+u2)**(-1) + 128*(t2*u2+s*m12)**(-2)*
     +    m1**4*u2 + 128*(t2*u2+s*m12)**(-2)*m1**6*u2*s4**(-1) + 128*
     +    (t2*u2+s*m12)**(-2)*m1**6*u2*t2s**(-1) - 128*
     +    (t2*u2+s*m12)**(-2)*m1**6*u2**2*t2s**(-2) + 128*
     +    (t2*u2+s*m12)**(-2)*m1**8*u2*s4**(-1)*t2s**(-1) - 128*
     +    (t2*u2+s*m12)**(-2)*m1**8*u2*t2s**(-2) - 128*
     +    (t2*u2+s*m12)**(-2)*m1**8*u2**2*s4**(-1)*t2s**(-2) + 128*
     +    (t2*u2+s*m12)**(-2)*m1**8*t2s**(-1) - 128*(t2*u2+s*m12)**(-2)
     +    *m1**10*u2*s4**(-1)*t2s**(-2) + 128*(t2*u2+s*m12)**(-2)*
     +    m1**10*s4**(-1)*t2s**(-1) + 128*(t2*u2+s*m12)**(-2)*m1**10*
     +    u2s**(-1)*t2s**(-1) - 128*(t2*u2+s*m12)**(-2)*m1**10*
     +    t2s**(-2) + 128*(t2*u2+s*m12)**(-2)*m1**12*s4**(-1)*u2s**(-1)
     +    *t2s**(-1) - 128*(t2*u2+s*m12)**(-2)*m1**12*s4**(-1)*
     +    t2s**(-2) )
      MM_qg = MM_qg + Ar2*Nc**(-1)*st**2*Pi**3*alphas**2*hardfac*
     + log_all * (  - 128*(t2*u2+s*m12)**(-2)*m1**12*u2s**(-1)*
     +    t2s**(-2) - 128*(t2*u2+s*m12)**(-2)*m1**14*s4**(-1)*u2s**(-1)
     +    *t2s**(-2) - 256*(t2*u2+s*m12)**(-1)*m1**2 + 256*
     +    (t2*u2+s*m12)**(-1)*m1**4*u2*t2s**(-2) - 256*
     +    (t2*u2+s*m12)**(-1)*m1**4*s4**(-1) - 128*(t2*u2+s*m12)**(-1)*
     +    m1**4*u2s**(-1) - 256*(t2*u2+s*m12)**(-1)*m1**4*t2s**(-1) + 
     +    256*(t2*u2+s*m12)**(-1)*m1**6*u2*s4**(-1)*t2s**(-2) - 384*
     +    (t2*u2+s*m12)**(-1)*m1**6*u2*t2s**(-3) - 128*
     +    (t2*u2+s*m12)**(-1)*m1**6*s4**(-1)*u2s**(-1) - 256*
     +    (t2*u2+s*m12)**(-1)*m1**6*s4**(-1)*t2s**(-1) - 384*
     +    (t2*u2+s*m12)**(-1)*m1**6*u2s**(-1)*t2s**(-1) + 384*
     +    (t2*u2+s*m12)**(-1)*m1**6*t2s**(-2) + 128*(t2*u2+s*m12)**(-1)
     +    *m1**8*t2**(-1)*u2*t2s**(-3) + 128*(t2*u2+s*m12)**(-1)*m1**8*
     +    t2**(-1)*u2s**(-1)*t2s**(-1) - 384*(t2*u2+s*m12)**(-1)*m1**8*
     +    u2*s4**(-1)*t2s**(-3) )
      MM_qg = MM_qg + Ar2*Nc**(-1)*st**2*Pi**3*alphas**2*hardfac*
     + log_all * (  - 384*(t2*u2+s*m12)**(-1)*m1**8*s4**(-1)*u2s**(-1)*
     +    t2s**(-1) + 384*(t2*u2+s*m12)**(-1)*m1**8*s4**(-1)*t2s**(-2)
     +     + 512*(t2*u2+s*m12)**(-1)*m1**8*u2s**(-1)*t2s**(-2) - 512*
     +    (t2*u2+s*m12)**(-1)*m1**8*t2s**(-3) + 128*(t2*u2+s*m12)**(-1)
     +    *m1**10*t2**(-1)*u2*s4**(-1)*t2s**(-3) + 128*
     +    (t2*u2+s*m12)**(-1)*m1**10*t2**(-1)*s4**(-1)*u2s**(-1)*
     +    t2s**(-1) + 128*(t2*u2+s*m12)**(-1)*m1**10*t2**(-1)*t2s**(-3)
     +     + 512*(t2*u2+s*m12)**(-1)*m1**10*s4**(-1)*u2s**(-1)*
     +    t2s**(-2) - 512*(t2*u2+s*m12)**(-1)*m1**10*s4**(-1)*t2s**(-3)
     +     + 128*(t2*u2+s*m12)**(-1)*m1**10*u2s**(-2)*t2s**(-2) - 640*
     +    (t2*u2+s*m12)**(-1)*m1**10*u2s**(-1)*t2s**(-3) + 128*
     +    (t2*u2+s*m12)**(-1)*m1**12*t2**(-1)*s4**(-1)*t2s**(-3) + 128*
     +    (t2*u2+s*m12)**(-1)*m1**12*t2**(-1)*u2s**(-1)*t2s**(-3) + 128
     +    *(t2*u2+s*m12)**(-1)*m1**12*s4**(-1)*u2s**(-2)*t2s**(-2) - 
     +    640*(t2*u2+s*m12)**(-1)*m1**12*s4**(-1)*u2s**(-1)*t2s**(-3) )
      MM_qg = MM_qg + Ar2*Nc**(-1)*st**2*Pi**3*alphas**2*hardfac*
     + log_all * (  - 128*(t2*u2+s*m12)**(-1)*m1**12*u2s**(-2)*
     +    t2s**(-3) + 128*(t2*u2+s*m12)**(-1)*m1**14*t2**(-1)*s4**(-1)*
     +    u2s**(-1)*t2s**(-3) - 128*(t2*u2+s*m12)**(-1)*m1**14*s4**(-1)
     +    *u2s**(-2)*t2s**(-3) + 128*m1**2*t2*u2*s4**(-2)*t2s**(-2) - 
     +    384*m1**2*u2*s4**(-1)*t2s**(-2) - 128*m1**2*s4**(-2) + 128*
     +    m1**2*s4**(-1)*u2s**(-1) + 256*m1**2*s4**(-1)*t2s**(-1) - 384
     +    *m1**4*u2*s4**(-2)*t2s**(-2) + 384*m1**4*u2*s4**(-1)*
     +    t2s**(-3) + 128*m1**4*s4**(-2)*u2s**(-1) + 256*m1**4*s4**(-2)
     +    *t2s**(-1) + 384*m1**4*s4**(-1)*u2s**(-1)*t2s**(-1) - 384*
     +    m1**4*s4**(-1)*t2s**(-2) + 384*m1**6*u2*s4**(-2)*t2s**(-3) + 
     +    384*m1**6*s4**(-2)*u2s**(-1)*t2s**(-1) - 384*m1**6*s4**(-2)*
     +    t2s**(-2) - 512*m1**6*s4**(-1)*u2s**(-1)*t2s**(-2) + 512*
     +    m1**6*s4**(-1)*t2s**(-3) - 512*m1**8*s4**(-2)*u2s**(-1)*
     +    t2s**(-2) + 512*m1**8*s4**(-2)*t2s**(-3) - 128*m1**8*s4**(-1)
     +    *u2s**(-2)*t2s**(-2) )
      MM_qg = MM_qg + Ar2*Nc**(-1)*st**2*Pi**3*alphas**2*hardfac*
     + log_all * ( 640*m1**8*s4**(-1)*u2s**(-1)*t2s**(-3) - 128*m1**10*
     +    s4**(-2)*u2s**(-2)*t2s**(-2) + 640*m1**10*s4**(-2)*u2s**(-1)*
     +    t2s**(-3) + 128*m1**10*s4**(-1)*u2s**(-2)*t2s**(-3) + 128*
     +    m1**12*s4**(-2)*u2s**(-2)*t2s**(-3) + 128*t2*u2*s4**(-1)*
     +    t2s**(-2) - 128*s4**(-1) )
      MM_qg = MM_qg + Ar2*Nc**(-1)*st**2*Pi**3*alphas**2*hardfac * ( 
     +     - 64*s**(-1)*m1**2*t2**(-1)*u2*s4**(-1) + 64*s**(-1)*m1**2*
     +    t2**(-1)*u2*t2s**(-1) + 64*s**(-1)*m1**2*t2**(-1) - 64*
     +    s**(-1)*m1**2*t2*s4**(-2) - 64*s**(-1)*m1**2*u2*s4**(-1)*
     +    t2s**(-1) - 128*s**(-1)*m1**2*u2*t2s**(-2) + 64*s**(-1)*m1**2
     +    *u2**2*s4**(-2)*t2s**(-1) + 64*s**(-1)*m1**2*u2**2*s4**(-1)*
     +    t2s**(-2) + 64*s**(-1)*m1**2*t2s**(-1) + 64*s**(-1)*m1**4*
     +    t2**(-1)*u2*s4**(-1)*t2s**(-1) + 64*s**(-1)*m1**4*t2**(-1)*
     +    s4**(-1) - 64*s**(-1)*m1**4*t2**(-1)*t2s**(-1) - 128*s**(-1)*
     +    m1**4*u2*s4**(-1)*t2s**(-2) + 64*s**(-1)*m1**4*u2**2*s4**(-2)
     +    *t2s**(-2) + 64*s**(-1)*m1**4*s4**(-1)*t2s**(-1) - 64*s**(-1)
     +    *m1**6*t2**(-1)*s4**(-1)*t2s**(-1) - 64*s**(-1)*t2**(-1)*u2
     +     - 64*s**(-1)*t2*s4**(-1) - 64*s**(-1)*u2*t2s**(-1) + 64*
     +    s**(-1)*u2**2*s4**(-1)*t2s**(-1) + 896*s*(t2*u2+s*m12)**(-2)*
     +    m1**2*t2 - 128*s*(t2*u2+s*m12)**(-2)*m1**2*t2**2*s4**(-1) + 
     +    2304*s*(t2*u2+s*m12)**(-2)*m1**2*u2 )
      MM_qg = MM_qg + Ar2*Nc**(-1)*st**2*Pi**3*alphas**2*hardfac * ( 
     +     - 640*s*(t2*u2+s*m12)**(-2)*m1**2*u2**2*s4**(-1) - 896*s*
     +    (t2*u2+s*m12)**(-2)*m1**2*s4 + 640*s*(t2*u2+s*m12)**(-2)*
     +    m1**4*t2*s4**(-1) + 1664*s*(t2*u2+s*m12)**(-2)*m1**4*u2*
     +    s4**(-1) - 1792*s*(t2*u2+s*m12)**(-2)*m1**4 - 1024*s*
     +    (t2*u2+s*m12)**(-2)*m1**6*s4**(-1) + 256*s*
     +    (t2*u2+s*m12)**(-2)*t2*s4 - 128*s*(t2*u2+s*m12)**(-2)*t2**2
     +     + 640*s*(t2*u2+s*m12)**(-2)*u2*s4 - 640*s*
     +    (t2*u2+s*m12)**(-2)*u2**2 - 128*s*(t2*u2+s*m12)**(-2)*s4**2
     +     - 64*s*(t2*u2+s*m12)**(-1)*m1**2*t2**(-1) - 832*s*
     +    (t2*u2+s*m12)**(-1)*m1**2*s4**(-1) - 64*s*(t2*u2+s*m12)**(-1)
     +    *m1**4*t2**(-1)*s4**(-1) - 832*s*(t2*u2+s*m12)**(-1) + 64*s*
     +    m1**2*s4**(-2)*t2s**(-1) + 64*s*m1**2*s4**(-1)*t2s**(-2) + 64
     +    *s*m1**4*s4**(-2)*t2s**(-2) + 64*s*s4**(-1)*t2s**(-1) - 256*
     +    s**2*(t2*u2+s*m12)**(-2)*m1**2*t2*s4**(-1) - 512*s**2*
     +    (t2*u2+s*m12)**(-2)*m1**2*u2*s4**(-1) )
      MM_qg = MM_qg + Ar2*Nc**(-1)*st**2*Pi**3*alphas**2*hardfac * ( 
     +    1408*s**2*(t2*u2+s*m12)**(-2)*m1**2 + 1152*s**2*
     +    (t2*u2+s*m12)**(-2)*m1**4*s4**(-1) - 256*s**2*
     +    (t2*u2+s*m12)**(-2)*t2 - 512*s**2*(t2*u2+s*m12)**(-2)*u2 + 
     +    256*s**2*(t2*u2+s*m12)**(-2)*s4 - 128*s**3*
     +    (t2*u2+s*m12)**(-2)*m1**2*s4**(-1) - 128*s**3*
     +    (t2*u2+s*m12)**(-2) - 640*(t2*u2+s*m12)**(-2)*m1**2*u2*s4 + 
     +    1152*(t2*u2+s*m12)**(-2)*m1**2*u2**2 - 256*
     +    (t2*u2+s*m12)**(-2)*m1**2*u2**3*s4**(-1) - 1024*
     +    (t2*u2+s*m12)**(-2)*m1**4*u2 + 768*(t2*u2+s*m12)**(-2)*m1**4*
     +    u2**2*s4**(-1) - 512*(t2*u2+s*m12)**(-2)*m1**6*u2*s4**(-1) - 
     +    128*(t2*u2+s*m12)**(-2)*u2*s4**2 + 384*(t2*u2+s*m12)**(-2)*
     +    u2**2*s4 - 256*(t2*u2+s*m12)**(-2)*u2**3 + 128*
     +    (t2*u2+s*m12)**(-1)*m1**2*t2**(-1)*u2 - 320*
     +    (t2*u2+s*m12)**(-1)*m1**2*t2*s4**(-1) - 640*
     +    (t2*u2+s*m12)**(-1)*m1**2*u2*s4**(-1) )
      MM_qg = MM_qg + Ar2*Nc**(-1)*st**2*Pi**3*alphas**2*hardfac * ( 
     +    1280*(t2*u2+s*m12)**(-1)*m1**2 + 128*(t2*u2+s*m12)**(-1)*
     +    m1**4*t2**(-1)*u2*s4**(-1) - 128*(t2*u2+s*m12)**(-1)*m1**4*
     +    t2**(-1) + 832*(t2*u2+s*m12)**(-1)*m1**4*s4**(-1) - 128*
     +    (t2*u2+s*m12)**(-1)*m1**6*t2**(-1)*s4**(-1) - 320*
     +    (t2*u2+s*m12)**(-1)*t2 - 640*(t2*u2+s*m12)**(-1)*u2 + 448*
     +    (t2*u2+s*m12)**(-1)*s4 - 64*m1**2*t2**(-1)*t2s**(-1) + 128*
     +    m1**2*u2*s4**(-2)*t2s**(-1) + 128*m1**2*u2*s4**(-1)*t2s**(-2)
     +     - 64*m1**2*s4**(-1)*t2s**(-1) - 128*m1**2*t2s**(-2) - 64*
     +    m1**4*t2**(-1)*s4**(-1)*t2s**(-1) + 128*m1**4*u2*s4**(-2)*
     +    t2s**(-2) - 128*m1**4*s4**(-1)*t2s**(-2) + 128*u2*s4**(-1)*
     +    t2s**(-1) - 64*t2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac*log_all * ( 
     +     - 256*s**(-1)*(s+t2)**(-1)*m1**2*s4**(-1)*u2s - 512*s**(-1)*
     +    (s+t2)**(-1)*m1**2 - 1024*s**(-1)*(s+t2)**(-1)*m1**4*s4**(-1)
     +     - 512*s**(-1)*(s+t2)**(-1)*m1**6*s4**(-2) - 256*s**(-1)*
     +    (s+t2)**(-1)*u2s - 256*s**(-1)*(s+u2)**(-1)*m1**2*t2*u2*
     +    s4**(-2) - 256*s**(-1)*(s+u2)**(-1)*t2*u2*s4**(-1) - 256*
     +    s**(-1)*m1**2*t2*u2**2*s4**(-2)*t2s**(-2) + 128*s**(-1)*m1**2
     +    *t2*s4**(-2) - 1024*s**(-1)*m1**2*t2*u2s**(-2) + 256*s**(-1)*
     +    m1**2*t2**2*s4**(-2)*u2s**(-1) + 256*s**(-1)*m1**2*t2**2*
     +    s4**(-1)*u2s**(-2) - 512*s**(-1)*m1**2*t2**2*u2s**(-3) + 128*
     +    s**(-1)*m1**2*u2**2*s4**(-2)*t2s**(-1) + 384*s**(-1)*m1**2*
     +    u2**2*s4**(-1)*t2s**(-2) + 512*s**(-1)*m1**2*s4**(-1) - 1024*
     +    s**(-1)*m1**2*u2s**(-1) + 512*s**(-1)*m1**4*t2*s4**(-2)*
     +    u2s**(-1) - 1024*s**(-1)*m1**4*t2*s4**(-1)*u2s**(-2) - 1024*
     +    s**(-1)*m1**4*t2*u2s**(-3) + 512*s**(-1)*m1**4*t2**2*s4**(-2)
     +    *u2s**(-2) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac*log_all * ( 
     +     - 512*s**(-1)*m1**4*t2**2*u2s**(-4) + 384*s**(-1)*m1**4*
     +    u2**2*s4**(-2)*t2s**(-2) - 256*s**(-1)*m1**4*u2**2*s4**(-1)*
     +    t2s**(-3) + 768*s**(-1)*m1**4*s4**(-2) - 1536*s**(-1)*m1**4*
     +    s4**(-1)*u2s**(-1) - 256*s**(-1)*m1**4*s4**(-1)*t2s**(-1) - 
     +    512*s**(-1)*m1**4*u2s**(-2) + 256*s**(-1)*m1**6*t2**(-1)*u2*
     +    t2s**(-3) + 256*s**(-1)*m1**6*t2**(-1)*u2s**(-1)*t2s**(-1) - 
     +    1024*s**(-1)*m1**6*t2*s4**(-1)*u2s**(-3) + 512*s**(-1)*m1**6*
     +    t2**2*s4**(-2)*u2s**(-3) - 512*s**(-1)*m1**6*t2**2*s4**(-1)*
     +    u2s**(-4) - 256*s**(-1)*m1**6*u2*s4**(-1)*t2s**(-3) - 256*
     +    s**(-1)*m1**6*u2**2*s4**(-2)*t2s**(-3) - 512*s**(-1)*m1**6*
     +    s4**(-2)*u2s**(-1) - 256*s**(-1)*m1**6*s4**(-2)*t2s**(-1) - 
     +    512*s**(-1)*m1**6*s4**(-1)*u2s**(-2) - 256*s**(-1)*m1**6*
     +    s4**(-1)*u2s**(-1)*t2s**(-1) + 256*s**(-1)*m1**8*t2**(-1)*u2*
     +    s4**(-1)*t2s**(-3) + 256*s**(-1)*m1**8*t2**(-1)*s4**(-1)*
     +    u2s**(-1)*t2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac*log_all * ( 
     +    256*s**(-1)*m1**8*t2**(-1)*t2s**(-3) - 256*s**(-1)*m1**8*u2*
     +    s4**(-2)*t2s**(-3) - 256*s**(-1)*m1**8*s4**(-2)*u2s**(-1)*
     +    t2s**(-1) - 256*s**(-1)*m1**8*s4**(-1)*t2s**(-3) + 256*
     +    s**(-1)*m1**10*t2**(-1)*s4**(-1)*t2s**(-3) + 256*s**(-1)*
     +    m1**10*t2**(-1)*u2s**(-1)*t2s**(-3) - 256*s**(-1)*m1**10*
     +    s4**(-2)*t2s**(-3) - 256*s**(-1)*m1**10*s4**(-1)*u2s**(-1)*
     +    t2s**(-3) + 256*s**(-1)*m1**12*t2**(-1)*s4**(-1)*u2s**(-1)*
     +    t2s**(-3) - 256*s**(-1)*m1**12*s4**(-2)*u2s**(-1)*t2s**(-3)
     +     - 256*s**(-1)*t2*u2**2*s4**(-1)*t2s**(-2) + 128*s**(-1)*t2*
     +    s4**(-1) - 512*s**(-1)*t2*u2s**(-1) + 256*s**(-1)*t2**2*
     +    s4**(-1)*u2s**(-1) - 256*s**(-1)*t2**2*u2s**(-2) + 128*
     +    s**(-1)*u2**2*s4**(-1)*t2s**(-1) - 256*s**(-1) - 256*s*
     +    (t2*u2+s*m12)**(-2)*m1**4 + 768*s*(t2*u2+s*m12)**(-2)*m1**6*
     +    u2*t2s**(-2) - 256*s*(t2*u2+s*m12)**(-2)*m1**6*s4**(-1) - 512
     +    *s*(t2*u2+s*m12)**(-2)*m1**6*t2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac*log_all * ( 
     +    768*s*(t2*u2+s*m12)**(-2)*m1**8*u2*s4**(-1)*t2s**(-2) - 512*s
     +    *(t2*u2+s*m12)**(-2)*m1**8*s4**(-1)*t2s**(-1) - 512*s*
     +    (t2*u2+s*m12)**(-2)*m1**8*u2s**(-1)*t2s**(-1) + 768*s*
     +    (t2*u2+s*m12)**(-2)*m1**8*t2s**(-2) - 512*s*
     +    (t2*u2+s*m12)**(-2)*m1**10*s4**(-1)*u2s**(-1)*t2s**(-1) + 768
     +    *s*(t2*u2+s*m12)**(-2)*m1**10*s4**(-1)*t2s**(-2) + 768*s*
     +    (t2*u2+s*m12)**(-2)*m1**10*u2s**(-1)*t2s**(-2) + 768*s*
     +    (t2*u2+s*m12)**(-2)*m1**12*s4**(-1)*u2s**(-1)*t2s**(-2) + 512
     +    *s*(t2*u2+s*m12)**(-1)*m1**4*u2s**(-1)*t2s**(-1) - 1024*s*
     +    (t2*u2+s*m12)**(-1)*m1**4*t2s**(-2) + 512*s*
     +    (t2*u2+s*m12)**(-1)*m1**6*s4**(-1)*u2s**(-1)*t2s**(-1) - 1024
     +    *s*(t2*u2+s*m12)**(-1)*m1**6*s4**(-1)*t2s**(-2) - 1536*s*
     +    (t2*u2+s*m12)**(-1)*m1**6*u2s**(-1)*t2s**(-2) + 768*s*
     +    (t2*u2+s*m12)**(-1)*m1**6*t2s**(-3) - 1536*s*
     +    (t2*u2+s*m12)**(-1)*m1**8*s4**(-1)*u2s**(-1)*t2s**(-2) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac*log_all * ( 
     +    768*s*(t2*u2+s*m12)**(-1)*m1**8*s4**(-1)*t2s**(-3) - 512*s*
     +    (t2*u2+s*m12)**(-1)*m1**8*u2s**(-2)*t2s**(-2) + 1536*s*
     +    (t2*u2+s*m12)**(-1)*m1**8*u2s**(-1)*t2s**(-3) - 512*s*
     +    (t2*u2+s*m12)**(-1)*m1**10*s4**(-1)*u2s**(-2)*t2s**(-2) + 
     +    1536*s*(t2*u2+s*m12)**(-1)*m1**10*s4**(-1)*u2s**(-1)*
     +    t2s**(-3) + 768*s*(t2*u2+s*m12)**(-1)*m1**10*u2s**(-2)*
     +    t2s**(-3) + 768*s*(t2*u2+s*m12)**(-1)*m1**12*s4**(-1)*
     +    u2s**(-2)*t2s**(-3) - 128*s*m1**2*s4**(-2)*t2s**(-1) - 512*s*
     +    m1**2*s4**(-1)*u2s**(-1)*t2s**(-1) + 640*s*m1**2*s4**(-1)*
     +    t2s**(-2) - 512*s*m1**4*s4**(-2)*u2s**(-1)*t2s**(-1) + 640*s*
     +    m1**4*s4**(-2)*t2s**(-2) + 1536*s*m1**4*s4**(-1)*u2s**(-1)*
     +    t2s**(-2) - 768*s*m1**4*s4**(-1)*t2s**(-3) + 1536*s*m1**6*
     +    s4**(-2)*u2s**(-1)*t2s**(-2) - 768*s*m1**6*s4**(-2)*t2s**(-3)
     +     + 512*s*m1**6*s4**(-1)*u2s**(-2)*t2s**(-2) - 1536*s*m1**6*
     +    s4**(-1)*u2s**(-1)*t2s**(-3) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac*log_all * ( 
     +    512*s*m1**8*s4**(-2)*u2s**(-2)*t2s**(-2) - 1536*s*m1**8*
     +    s4**(-2)*u2s**(-1)*t2s**(-3) - 768*s*m1**8*s4**(-1)*u2s**(-2)
     +    *t2s**(-3) - 768*s*m1**10*s4**(-2)*u2s**(-2)*t2s**(-3) - 128*
     +    s*s4**(-1)*t2s**(-1) - 256*s**2*(t2*u2+s*m12)**(-2)*m1**6*
     +    u2s**(-1)*t2s**(-1) + 768*s**2*(t2*u2+s*m12)**(-2)*m1**6*
     +    t2s**(-2) - 256*s**2*(t2*u2+s*m12)**(-2)*m1**8*s4**(-1)*
     +    u2s**(-1)*t2s**(-1) + 768*s**2*(t2*u2+s*m12)**(-2)*m1**8*
     +    s4**(-1)*t2s**(-2) + 768*s**2*(t2*u2+s*m12)**(-2)*m1**8*
     +    u2s**(-1)*t2s**(-2) + 768*s**2*(t2*u2+s*m12)**(-2)*m1**10*
     +    s4**(-1)*u2s**(-1)*t2s**(-2) - 256*s**2*(t2*u2+s*m12)**(-1)*
     +    m1**4*u2s**(-1)*t2s**(-2) - 256*s**2*(t2*u2+s*m12)**(-1)*
     +    m1**6*s4**(-1)*u2s**(-1)*t2s**(-2) - 256*s**2*
     +    (t2*u2+s*m12)**(-1)*m1**6*u2s**(-2)*t2s**(-2) + 512*s**2*
     +    (t2*u2+s*m12)**(-1)*m1**6*u2s**(-1)*t2s**(-3) - 256*s**2*
     +    (t2*u2+s*m12)**(-1)*m1**8*s4**(-1)*u2s**(-2)*t2s**(-2) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac*log_all * ( 
     +    512*s**2*(t2*u2+s*m12)**(-1)*m1**8*s4**(-1)*u2s**(-1)*
     +    t2s**(-3) + 768*s**2*(t2*u2+s*m12)**(-1)*m1**8*u2s**(-2)*
     +    t2s**(-3) + 768*s**2*(t2*u2+s*m12)**(-1)*m1**10*s4**(-1)*
     +    u2s**(-2)*t2s**(-3) + 256*s**2*m1**2*s4**(-1)*u2s**(-1)*
     +    t2s**(-2) + 256*s**2*m1**4*s4**(-2)*u2s**(-1)*t2s**(-2) + 256
     +    *s**2*m1**4*s4**(-1)*u2s**(-2)*t2s**(-2) - 512*s**2*m1**4*
     +    s4**(-1)*u2s**(-1)*t2s**(-3) + 256*s**2*m1**6*s4**(-2)*
     +    u2s**(-2)*t2s**(-2) - 512*s**2*m1**6*s4**(-2)*u2s**(-1)*
     +    t2s**(-3) - 768*s**2*m1**6*s4**(-1)*u2s**(-2)*t2s**(-3) - 768
     +    *s**2*m1**8*s4**(-2)*u2s**(-2)*t2s**(-3) + 256*s**3*
     +    (t2*u2+s*m12)**(-2)*m1**6*u2s**(-1)*t2s**(-2) + 256*s**3*
     +    (t2*u2+s*m12)**(-2)*m1**8*s4**(-1)*u2s**(-1)*t2s**(-2) + 256*
     +    s**3*(t2*u2+s*m12)**(-1)*m1**6*u2s**(-2)*t2s**(-3) + 256*s**3
     +    *(t2*u2+s*m12)**(-1)*m1**8*s4**(-1)*u2s**(-2)*t2s**(-3) - 256
     +    *s**3*m1**4*s4**(-1)*u2s**(-2)*t2s**(-3) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac*log_all * ( 
     +     - 256*s**3*m1**6*s4**(-2)*u2s**(-2)*t2s**(-3) + 256*
     +    (s+t2)**(-2)*m1**2*s4**(-1)*u2s + 512*(s+t2)**(-2)*m1**2 + 
     +    1024*(s+t2)**(-2)*m1**4*s4**(-1) + 512*(s+t2)**(-2)*m1**6*
     +    s4**(-2) + 256*(s+t2)**(-2)*u2s + 768*(s+t2)**(-1)*m1**2*
     +    s4**(-1) + 1536*(s+t2)**(-1)*m1**4*s4**(-2) + 1024*
     +    (s+t2)**(-1)*m1**6*s4**(-3) + 256*(s+t2)**(-1) - 512*
     +    (s+u2)**(-1)*m1**2*s4**(-1) - 256*(s+u2)**(-1)*m1**4*s4**(-2)
     +     - 256*(s+u2)**(-1) - 256*(t2*u2+s*m12)**(-2)*m1**4*u2 - 256*
     +    (t2*u2+s*m12)**(-2)*m1**6*u2*s4**(-1) - 256*
     +    (t2*u2+s*m12)**(-2)*m1**6*u2*t2s**(-1) + 256*
     +    (t2*u2+s*m12)**(-2)*m1**6*u2**2*t2s**(-2) - 256*
     +    (t2*u2+s*m12)**(-2)*m1**8*u2*s4**(-1)*t2s**(-1) + 256*
     +    (t2*u2+s*m12)**(-2)*m1**8*u2*t2s**(-2) + 256*
     +    (t2*u2+s*m12)**(-2)*m1**8*u2**2*s4**(-1)*t2s**(-2) - 256*
     +    (t2*u2+s*m12)**(-2)*m1**8*t2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac*log_all * ( 
     +    256*(t2*u2+s*m12)**(-2)*m1**10*u2*s4**(-1)*t2s**(-2) - 256*
     +    (t2*u2+s*m12)**(-2)*m1**10*s4**(-1)*t2s**(-1) - 256*
     +    (t2*u2+s*m12)**(-2)*m1**10*u2s**(-1)*t2s**(-1) + 256*
     +    (t2*u2+s*m12)**(-2)*m1**10*t2s**(-2) - 256*
     +    (t2*u2+s*m12)**(-2)*m1**12*s4**(-1)*u2s**(-1)*t2s**(-1) + 256
     +    *(t2*u2+s*m12)**(-2)*m1**12*s4**(-1)*t2s**(-2) + 256*
     +    (t2*u2+s*m12)**(-2)*m1**12*u2s**(-1)*t2s**(-2) + 256*
     +    (t2*u2+s*m12)**(-2)*m1**14*s4**(-1)*u2s**(-1)*t2s**(-2) + 512
     +    *(t2*u2+s*m12)**(-1)*m1**2 - 512*(t2*u2+s*m12)**(-1)*m1**4*u2
     +    *t2s**(-2) + 512*(t2*u2+s*m12)**(-1)*m1**4*s4**(-1) + 256*
     +    (t2*u2+s*m12)**(-1)*m1**4*u2s**(-1) + 512*(t2*u2+s*m12)**(-1)
     +    *m1**4*t2s**(-1) - 512*(t2*u2+s*m12)**(-1)*m1**6*u2*s4**(-1)*
     +    t2s**(-2) + 768*(t2*u2+s*m12)**(-1)*m1**6*u2*t2s**(-3) + 256*
     +    (t2*u2+s*m12)**(-1)*m1**6*s4**(-1)*u2s**(-1) + 512*
     +    (t2*u2+s*m12)**(-1)*m1**6*s4**(-1)*t2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac*log_all * ( 
     +    768*(t2*u2+s*m12)**(-1)*m1**6*u2s**(-1)*t2s**(-1) - 768*
     +    (t2*u2+s*m12)**(-1)*m1**6*t2s**(-2) - 256*(t2*u2+s*m12)**(-1)
     +    *m1**8*t2**(-1)*u2*t2s**(-3) - 256*(t2*u2+s*m12)**(-1)*m1**8*
     +    t2**(-1)*u2s**(-1)*t2s**(-1) + 768*(t2*u2+s*m12)**(-1)*m1**8*
     +    u2*s4**(-1)*t2s**(-3) + 768*(t2*u2+s*m12)**(-1)*m1**8*
     +    s4**(-1)*u2s**(-1)*t2s**(-1) - 768*(t2*u2+s*m12)**(-1)*m1**8*
     +    s4**(-1)*t2s**(-2) - 1024*(t2*u2+s*m12)**(-1)*m1**8*u2s**(-1)
     +    *t2s**(-2) + 1024*(t2*u2+s*m12)**(-1)*m1**8*t2s**(-3) - 256*
     +    (t2*u2+s*m12)**(-1)*m1**10*t2**(-1)*u2*s4**(-1)*t2s**(-3) - 
     +    256*(t2*u2+s*m12)**(-1)*m1**10*t2**(-1)*s4**(-1)*u2s**(-1)*
     +    t2s**(-1) - 256*(t2*u2+s*m12)**(-1)*m1**10*t2**(-1)*t2s**(-3)
     +     - 1024*(t2*u2+s*m12)**(-1)*m1**10*s4**(-1)*u2s**(-1)*
     +    t2s**(-2) + 1024*(t2*u2+s*m12)**(-1)*m1**10*s4**(-1)*
     +    t2s**(-3) - 256*(t2*u2+s*m12)**(-1)*m1**10*u2s**(-2)*
     +    t2s**(-2) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac*log_all * ( 
     +    1280*(t2*u2+s*m12)**(-1)*m1**10*u2s**(-1)*t2s**(-3) - 256*
     +    (t2*u2+s*m12)**(-1)*m1**12*t2**(-1)*s4**(-1)*t2s**(-3) - 256*
     +    (t2*u2+s*m12)**(-1)*m1**12*t2**(-1)*u2s**(-1)*t2s**(-3) - 256
     +    *(t2*u2+s*m12)**(-1)*m1**12*s4**(-1)*u2s**(-2)*t2s**(-2) + 
     +    1280*(t2*u2+s*m12)**(-1)*m1**12*s4**(-1)*u2s**(-1)*t2s**(-3)
     +     + 256*(t2*u2+s*m12)**(-1)*m1**12*u2s**(-2)*t2s**(-3) - 256*
     +    (t2*u2+s*m12)**(-1)*m1**14*t2**(-1)*s4**(-1)*u2s**(-1)*
     +    t2s**(-3) + 256*(t2*u2+s*m12)**(-1)*m1**14*s4**(-1)*u2s**(-2)
     +    *t2s**(-3) - 256*m1**2*t2*u2*s4**(-2)*t2s**(-2) + 256*m1**2*
     +    t2*s4**(-2)*u2s**(-1) + 256*m1**2*t2*s4**(-1)*u2s**(-2) - 512
     +    *m1**2*t2*u2s**(-3) + 768*m1**2*u2*s4**(-1)*t2s**(-2) + 256*
     +    m1**2*s4**(-2) + 768*m1**2*s4**(-1)*u2s**(-1) - 512*m1**2*
     +    s4**(-1)*t2s**(-1) + 512*m1**4*t2*s4**(-2)*u2s**(-2) - 512*
     +    m1**4*t2*u2s**(-4) + 768*m1**4*u2*s4**(-2)*t2s**(-2) - 768*
     +    m1**4*u2*s4**(-1)*t2s**(-3) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac*log_all * ( 
     +    1792*m1**4*s4**(-2)*u2s**(-1) - 512*m1**4*s4**(-2)*t2s**(-1)
     +     + 512*m1**4*s4**(-1)*u2s**(-2) - 768*m1**4*s4**(-1)*
     +    u2s**(-1)*t2s**(-1) + 768*m1**4*s4**(-1)*t2s**(-2) + 512*
     +    m1**6*t2*s4**(-2)*u2s**(-3) - 512*m1**6*t2*s4**(-1)*u2s**(-4)
     +     - 768*m1**6*u2*s4**(-2)*t2s**(-3) + 1024*m1**6*s4**(-3)*
     +    u2s**(-1) + 512*m1**6*s4**(-2)*u2s**(-2) - 768*m1**6*s4**(-2)
     +    *u2s**(-1)*t2s**(-1) + 768*m1**6*s4**(-2)*t2s**(-2) + 1024*
     +    m1**6*s4**(-1)*u2s**(-1)*t2s**(-2) - 1024*m1**6*s4**(-1)*
     +    t2s**(-3) + 1024*m1**8*s4**(-2)*u2s**(-1)*t2s**(-2) - 1024*
     +    m1**8*s4**(-2)*t2s**(-3) + 256*m1**8*s4**(-1)*u2s**(-2)*
     +    t2s**(-2) - 1280*m1**8*s4**(-1)*u2s**(-1)*t2s**(-3) + 256*
     +    m1**10*s4**(-2)*u2s**(-2)*t2s**(-2) - 1280*m1**10*s4**(-2)*
     +    u2s**(-1)*t2s**(-3) - 256*m1**10*s4**(-1)*u2s**(-2)*t2s**(-3)
     +     - 256*m1**12*s4**(-2)*u2s**(-2)*t2s**(-3) - 256*t2*u2*
     +    s4**(-1)*t2s**(-2) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac*log_all * ( 
     +    256*t2*s4**(-1)*u2s**(-1) - 256*t2*u2s**(-2) + 256*s4**(-1) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac * ( 128*
     +    s**(-1)*(s+t2)**(-1)*m1**2*u2*s4**(-1) + 128*s**(-1)*
     +    (s+t2)**(-1)*m1**2*u2**2*s4**(-2) - 512*s**(-1)*(s+t2)**(-1)*
     +    m1**2*s4**(-1)*u2s - 384*s**(-1)*(s+t2)**(-1)*m1**2 - 256*
     +    s**(-1)*(s+t2)**(-1)*m1**4*u2*s4**(-2) - 256*s**(-1)*
     +    (s+t2)**(-1)*m1**4*s4**(-1) + 128*s**(-1)*(s+t2)**(-1)*m1**6*
     +    s4**(-2) + 384*s**(-1)*(s+t2)**(-1)*u2 + 128*s**(-1)*
     +    (s+t2)**(-1)*u2**2*s4**(-1) - 512*s**(-1)*(s+t2)**(-1)*u2s + 
     +    128*s**(-1)*m1**2*t2**(-1)*u2*s4**(-1) - 128*s**(-1)*m1**2*
     +    t2**(-1)*u2*t2s**(-1) - 128*s**(-1)*m1**2*t2**(-1) + 224*
     +    s**(-1)*m1**2*t2*s4**(-2) - 192*s**(-1)*m1**2*t2*s4**(-1)*
     +    u2s**(-1) + 512*s**(-1)*m1**2*t2*u2s**(-2) + 192*s**(-1)*
     +    m1**2*t2**2*s4**(-2)*u2s**(-1) + 512*s**(-1)*m1**2*t2**2*
     +    u2s**(-3) + 32*s**(-1)*m1**2*u2*s4**(-2) + 128*s**(-1)*m1**2*
     +    u2*s4**(-1)*t2s**(-1) + 256*s**(-1)*m1**2*u2*t2s**(-2) - 128*
     +    s**(-1)*m1**2*u2**2*s4**(-2)*t2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac * (  - 128*
     +    s**(-1)*m1**2*u2**2*s4**(-1)*t2s**(-2) + 64*s**(-1)*m1**2*
     +    s4**(-1) - 128*s**(-1)*m1**2*t2s**(-1) - 128*s**(-1)*m1**4*
     +    t2**(-1)*u2*s4**(-1)*t2s**(-1) - 128*s**(-1)*m1**4*t2**(-1)*
     +    s4**(-1) + 128*s**(-1)*m1**4*t2**(-1)*t2s**(-1) + 512*s**(-1)
     +    *m1**4*t2*s4**(-1)*u2s**(-2) + 512*s**(-1)*m1**4*t2*u2s**(-3)
     +     + 512*s**(-1)*m1**4*t2**2*s4**(-1)*u2s**(-3) + 512*s**(-1)*
     +    m1**4*t2**2*u2s**(-4) + 256*s**(-1)*m1**4*u2*s4**(-1)*
     +    t2s**(-2) - 128*s**(-1)*m1**4*u2**2*s4**(-2)*t2s**(-2) - 32*
     +    s**(-1)*m1**4*s4**(-2) - 128*s**(-1)*m1**4*s4**(-1)*t2s**(-1)
     +     + 128*s**(-1)*m1**6*t2**(-1)*s4**(-1)*t2s**(-1) + 512*
     +    s**(-1)*m1**6*t2*s4**(-1)*u2s**(-3) + 512*s**(-1)*m1**6*t2**2
     +    *s4**(-1)*u2s**(-4) + 128*s**(-1)*t2**(-1)*u2 + 224*s**(-1)*
     +    t2*s4**(-1) - 192*s**(-1)*t2*u2s**(-1) + 192*s**(-1)*t2**2*
     +    s4**(-1)*u2s**(-1) + 32*s**(-1)*u2*s4**(-1) + 128*s**(-1)*u2*
     +    t2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac * (  - 128*
     +    s**(-1)*u2**2*s4**(-1)*t2s**(-1) + 96*s**(-1) - 1792*s*
     +    (t2*u2+s*m12)**(-2)*m1**2*t2 + 256*s*(t2*u2+s*m12)**(-2)*
     +    m1**2*t2**2*s4**(-1) - 4608*s*(t2*u2+s*m12)**(-2)*m1**2*u2 + 
     +    1280*s*(t2*u2+s*m12)**(-2)*m1**2*u2**2*s4**(-1) + 1792*s*
     +    (t2*u2+s*m12)**(-2)*m1**2*s4 - 1280*s*(t2*u2+s*m12)**(-2)*
     +    m1**4*t2*s4**(-1) - 3328*s*(t2*u2+s*m12)**(-2)*m1**4*u2*
     +    s4**(-1) + 3584*s*(t2*u2+s*m12)**(-2)*m1**4 + 2048*s*
     +    (t2*u2+s*m12)**(-2)*m1**6*s4**(-1) - 512*s*
     +    (t2*u2+s*m12)**(-2)*t2*s4 + 256*s*(t2*u2+s*m12)**(-2)*t2**2
     +     - 1280*s*(t2*u2+s*m12)**(-2)*u2*s4 + 1280*s*
     +    (t2*u2+s*m12)**(-2)*u2**2 + 256*s*(t2*u2+s*m12)**(-2)*s4**2
     +     + 128*s*(t2*u2+s*m12)**(-1)*m1**2*t2**(-1) + 1664*s*
     +    (t2*u2+s*m12)**(-1)*m1**2*s4**(-1) + 64*s*(t2*u2+s*m12)**(-1)
     +    *m1**2*u2s**(-1) + 128*s*(t2*u2+s*m12)**(-1)*m1**4*t2**(-1)*
     +    s4**(-1) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac * ( 64*s*
     +    (t2*u2+s*m12)**(-1)*m1**4*s4**(-1)*u2s**(-1) + 1664*s*
     +    (t2*u2+s*m12)**(-1) - 64*s*m1**2*t2**(-1)*s4**(-1)*u2s**(-1)
     +     - 256*s*m1**2*s4**(-2)*u2s**(-1) - 128*s*m1**2*s4**(-2)*
     +    t2s**(-1) - 256*s*m1**2*s4**(-1)*u2s**(-2) - 128*s*m1**2*
     +    s4**(-1)*t2s**(-2) - 256*s*m1**4*s4**(-2)*u2s**(-2) - 128*s*
     +    m1**4*s4**(-2)*t2s**(-2) - 64*s*t2**(-1)*u2s**(-1) - 256*s*
     +    s4**(-1)*u2s**(-1) - 128*s*s4**(-1)*t2s**(-1) + 512*s**2*
     +    (t2*u2+s*m12)**(-2)*m1**2*t2*s4**(-1) + 1024*s**2*
     +    (t2*u2+s*m12)**(-2)*m1**2*u2*s4**(-1) - 2816*s**2*
     +    (t2*u2+s*m12)**(-2)*m1**2 - 2304*s**2*(t2*u2+s*m12)**(-2)*
     +    m1**4*s4**(-1) + 512*s**2*(t2*u2+s*m12)**(-2)*t2 + 1024*s**2*
     +    (t2*u2+s*m12)**(-2)*u2 - 512*s**2*(t2*u2+s*m12)**(-2)*s4 + 64
     +    *s**2*(t2*u2+s*m12)**(-1)*m1**2*t2**(-1)*u2s**(-1) + 64*s**2*
     +    (t2*u2+s*m12)**(-1)*m1**4*t2**(-1)*s4**(-1)*u2s**(-1) + 256*
     +    s**3*(t2*u2+s*m12)**(-2)*m1**2*s4**(-1) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac * ( 256*s**3
     +    *(t2*u2+s*m12)**(-2) - 256*(s+t2)**(-2)*m1**2*u2*s4**(-1) + 
     +    256*(s+t2)**(-2)*m1**2*s4**(-1)*u2s + 256*(s+t2)**(-2)*m1**2
     +     + 256*(s+t2)**(-2)*m1**4*s4**(-1) - 256*(s+t2)**(-2)*u2 + 
     +    256*(s+t2)**(-2)*u2s + 320*(s+t2)**(-1)*m1**2*u2*s4**(-2) - 
     +    128*(s+t2)**(-1)*m1**2*u2**2*s4**(-3) - 128*(s+t2)**(-1)*
     +    m1**2*s4**(-1) + 128*(s+t2)**(-1)*m1**4*u2*s4**(-3) - 64*
     +    (s+t2)**(-1)*m1**4*s4**(-2) + 192*(s+t2)**(-1)*u2*s4**(-1) - 
     +    128*(s+t2)**(-1)*u2**2*s4**(-2) - 64*(s+t2)**(-1) + 1280*
     +    (t2*u2+s*m12)**(-2)*m1**2*u2*s4 - 2304*(t2*u2+s*m12)**(-2)*
     +    m1**2*u2**2 + 512*(t2*u2+s*m12)**(-2)*m1**2*u2**3*s4**(-1) + 
     +    2048*(t2*u2+s*m12)**(-2)*m1**4*u2 - 1536*(t2*u2+s*m12)**(-2)*
     +    m1**4*u2**2*s4**(-1) + 1024*(t2*u2+s*m12)**(-2)*m1**6*u2*
     +    s4**(-1) + 256*(t2*u2+s*m12)**(-2)*u2*s4**2 - 768*
     +    (t2*u2+s*m12)**(-2)*u2**2*s4 + 512*(t2*u2+s*m12)**(-2)*u2**3
     +     - 256*(t2*u2+s*m12)**(-1)*m1**2*t2**(-1)*u2 )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac * ( 576*
     +    (t2*u2+s*m12)**(-1)*m1**2*t2*s4**(-1) + 1216*
     +    (t2*u2+s*m12)**(-1)*m1**2*u2*s4**(-1) - 2432*
     +    (t2*u2+s*m12)**(-1)*m1**2 - 256*(t2*u2+s*m12)**(-1)*m1**4*
     +    t2**(-1)*u2*s4**(-1) + 256*(t2*u2+s*m12)**(-1)*m1**4*t2**(-1)
     +     - 1600*(t2*u2+s*m12)**(-1)*m1**4*s4**(-1) + 256*
     +    (t2*u2+s*m12)**(-1)*m1**6*t2**(-1)*s4**(-1) + 576*
     +    (t2*u2+s*m12)**(-1)*t2 + 1216*(t2*u2+s*m12)**(-1)*u2 - 832*
     +    (t2*u2+s*m12)**(-1)*s4 + 128*m1**2*t2**(-1)*t2s**(-1) - 64*
     +    m1**2*t2*s4**(-2)*u2s**(-1) - 256*m1**2*t2*s4**(-1)*u2s**(-2)
     +     + 512*m1**2*t2*u2s**(-3) - 128*m1**2*u2*s4**(-3) - 256*m1**2
     +    *u2*s4**(-2)*t2s**(-1) - 256*m1**2*u2*s4**(-1)*t2s**(-2) - 
     +    288*m1**2*s4**(-2) + 128*m1**2*s4**(-1)*t2s**(-1) + 256*m1**2
     +    *u2s**(-2) + 256*m1**2*t2s**(-2) + 128*m1**4*t2**(-1)*
     +    s4**(-1)*t2s**(-1) - 256*m1**4*t2*s4**(-2)*u2s**(-2) + 512*
     +    m1**4*t2*s4**(-1)*u2s**(-3) )
      MM_qg = MM_qg + Ar2*Nc*st**2*Pi**3*alphas**2*hardfac * ( 512*
     +    m1**4*t2*u2s**(-4) - 256*m1**4*u2*s4**(-2)*t2s**(-2) - 256*
     +    m1**4*s4**(-2)*u2s**(-1) + 256*m1**4*s4**(-1)*u2s**(-2) + 256
     +    *m1**4*s4**(-1)*t2s**(-2) + 512*m1**6*t2*s4**(-1)*u2s**(-4)
     +     - 64*t2*s4**(-1)*u2s**(-1) - 128*u2*s4**(-2) - 256*u2*
     +    s4**(-1)*t2s**(-1) - 288*s4**(-1) + 256*u2s**(-1) + 128*
     +    t2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac*log_all
     +  * ( 256*s**(-1)*(s+t2)**(-1)*m1**2*s4**(-1)*u2s + 512*s**(-1)*
     +    (s+t2)**(-1)*m1**2 + 1024*s**(-1)*(s+t2)**(-1)*m1**4*s4**(-1)
     +     + 512*s**(-1)*(s+t2)**(-1)*m1**6*s4**(-2) + 256*s**(-1)*
     +    (s+t2)**(-1)*u2s + 128*s**(-1)*(s+u2)**(-1)*m1**2*t2*u2*
     +    s4**(-2) + 128*s**(-1)*(s+u2)**(-1)*t2*u2*s4**(-1) + 128*
     +    s**(-1)*m1**2*t2*u2**2*s4**(-2)*t2s**(-2) - 64*s**(-1)*m1**2*
     +    t2*s4**(-2) + 1024*s**(-1)*m1**2*t2*u2s**(-2) - 256*s**(-1)*
     +    m1**2*t2**2*s4**(-2)*u2s**(-1) - 256*s**(-1)*m1**2*t2**2*
     +    s4**(-1)*u2s**(-2) + 512*s**(-1)*m1**2*t2**2*u2s**(-3) - 64*
     +    s**(-1)*m1**2*u2**2*s4**(-2)*t2s**(-1) - 192*s**(-1)*m1**2*
     +    u2**2*s4**(-1)*t2s**(-2) - 384*s**(-1)*m1**2*s4**(-1) + 1024*
     +    s**(-1)*m1**2*u2s**(-1) - 512*s**(-1)*m1**4*t2*s4**(-2)*
     +    u2s**(-1) + 1024*s**(-1)*m1**4*t2*s4**(-1)*u2s**(-2) + 1024*
     +    s**(-1)*m1**4*t2*u2s**(-3) - 512*s**(-1)*m1**4*t2**2*s4**(-2)
     +    *u2s**(-2) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac*log_all
     +  * ( 512*s**(-1)*m1**4*t2**2*u2s**(-4) - 192*s**(-1)*m1**4*u2**2
     +    *s4**(-2)*t2s**(-2) + 128*s**(-1)*m1**4*u2**2*s4**(-1)*
     +    t2s**(-3) - 640*s**(-1)*m1**4*s4**(-2) + 1536*s**(-1)*m1**4*
     +    s4**(-1)*u2s**(-1) + 128*s**(-1)*m1**4*s4**(-1)*t2s**(-1) + 
     +    512*s**(-1)*m1**4*u2s**(-2) - 128*s**(-1)*m1**6*t2**(-1)*u2*
     +    t2s**(-3) - 128*s**(-1)*m1**6*t2**(-1)*u2s**(-1)*t2s**(-1) + 
     +    1024*s**(-1)*m1**6*t2*s4**(-1)*u2s**(-3) - 512*s**(-1)*m1**6*
     +    t2**2*s4**(-2)*u2s**(-3) + 512*s**(-1)*m1**6*t2**2*s4**(-1)*
     +    u2s**(-4) + 128*s**(-1)*m1**6*u2*s4**(-1)*t2s**(-3) + 128*
     +    s**(-1)*m1**6*u2**2*s4**(-2)*t2s**(-3) + 512*s**(-1)*m1**6*
     +    s4**(-2)*u2s**(-1) + 128*s**(-1)*m1**6*s4**(-2)*t2s**(-1) + 
     +    512*s**(-1)*m1**6*s4**(-1)*u2s**(-2) + 128*s**(-1)*m1**6*
     +    s4**(-1)*u2s**(-1)*t2s**(-1) - 128*s**(-1)*m1**8*t2**(-1)*u2*
     +    s4**(-1)*t2s**(-3) - 128*s**(-1)*m1**8*t2**(-1)*s4**(-1)*
     +    u2s**(-1)*t2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac*log_all
     +  * (  - 128*s**(-1)*m1**8*t2**(-1)*t2s**(-3) + 128*s**(-1)*m1**8
     +    *u2*s4**(-2)*t2s**(-3) + 128*s**(-1)*m1**8*s4**(-2)*u2s**(-1)
     +    *t2s**(-1) + 128*s**(-1)*m1**8*s4**(-1)*t2s**(-3) - 128*
     +    s**(-1)*m1**10*t2**(-1)*s4**(-1)*t2s**(-3) - 128*s**(-1)*
     +    m1**10*t2**(-1)*u2s**(-1)*t2s**(-3) + 128*s**(-1)*m1**10*
     +    s4**(-2)*t2s**(-3) + 128*s**(-1)*m1**10*s4**(-1)*u2s**(-1)*
     +    t2s**(-3) - 128*s**(-1)*m1**12*t2**(-1)*s4**(-1)*u2s**(-1)*
     +    t2s**(-3) + 128*s**(-1)*m1**12*s4**(-2)*u2s**(-1)*t2s**(-3)
     +     + 128*s**(-1)*t2*u2**2*s4**(-1)*t2s**(-2) - 64*s**(-1)*t2*
     +    s4**(-1) + 512*s**(-1)*t2*u2s**(-1) - 256*s**(-1)*t2**2*
     +    s4**(-1)*u2s**(-1) + 256*s**(-1)*t2**2*u2s**(-2) - 64*s**(-1)
     +    *u2**2*s4**(-1)*t2s**(-1) + 256*s**(-1) + 128*s*
     +    (t2*u2+s*m12)**(-2)*m1**4 - 384*s*(t2*u2+s*m12)**(-2)*m1**6*
     +    u2*t2s**(-2) + 128*s*(t2*u2+s*m12)**(-2)*m1**6*s4**(-1) + 256
     +    *s*(t2*u2+s*m12)**(-2)*m1**6*t2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac*log_all
     +  * (  - 384*s*(t2*u2+s*m12)**(-2)*m1**8*u2*s4**(-1)*t2s**(-2) + 
     +    256*s*(t2*u2+s*m12)**(-2)*m1**8*s4**(-1)*t2s**(-1) + 256*s*
     +    (t2*u2+s*m12)**(-2)*m1**8*u2s**(-1)*t2s**(-1) - 384*s*
     +    (t2*u2+s*m12)**(-2)*m1**8*t2s**(-2) + 256*s*
     +    (t2*u2+s*m12)**(-2)*m1**10*s4**(-1)*u2s**(-1)*t2s**(-1) - 384
     +    *s*(t2*u2+s*m12)**(-2)*m1**10*s4**(-1)*t2s**(-2) - 384*s*
     +    (t2*u2+s*m12)**(-2)*m1**10*u2s**(-1)*t2s**(-2) - 384*s*
     +    (t2*u2+s*m12)**(-2)*m1**12*s4**(-1)*u2s**(-1)*t2s**(-2) - 256
     +    *s*(t2*u2+s*m12)**(-1)*m1**4*u2s**(-1)*t2s**(-1) + 512*s*
     +    (t2*u2+s*m12)**(-1)*m1**4*t2s**(-2) - 256*s*
     +    (t2*u2+s*m12)**(-1)*m1**6*s4**(-1)*u2s**(-1)*t2s**(-1) + 512*
     +    s*(t2*u2+s*m12)**(-1)*m1**6*s4**(-1)*t2s**(-2) + 768*s*
     +    (t2*u2+s*m12)**(-1)*m1**6*u2s**(-1)*t2s**(-2) - 384*s*
     +    (t2*u2+s*m12)**(-1)*m1**6*t2s**(-3) + 768*s*
     +    (t2*u2+s*m12)**(-1)*m1**8*s4**(-1)*u2s**(-1)*t2s**(-2) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac*log_all
     +  * (  - 384*s*(t2*u2+s*m12)**(-1)*m1**8*s4**(-1)*t2s**(-3) + 256
     +    *s*(t2*u2+s*m12)**(-1)*m1**8*u2s**(-2)*t2s**(-2) - 768*s*
     +    (t2*u2+s*m12)**(-1)*m1**8*u2s**(-1)*t2s**(-3) + 256*s*
     +    (t2*u2+s*m12)**(-1)*m1**10*s4**(-1)*u2s**(-2)*t2s**(-2) - 768
     +    *s*(t2*u2+s*m12)**(-1)*m1**10*s4**(-1)*u2s**(-1)*t2s**(-3) - 
     +    384*s*(t2*u2+s*m12)**(-1)*m1**10*u2s**(-2)*t2s**(-3) - 384*s*
     +    (t2*u2+s*m12)**(-1)*m1**12*s4**(-1)*u2s**(-2)*t2s**(-3) + 64*
     +    s*m1**2*s4**(-2)*t2s**(-1) + 256*s*m1**2*s4**(-1)*u2s**(-1)*
     +    t2s**(-1) - 320*s*m1**2*s4**(-1)*t2s**(-2) + 256*s*m1**4*
     +    s4**(-2)*u2s**(-1)*t2s**(-1) - 320*s*m1**4*s4**(-2)*t2s**(-2)
     +     - 768*s*m1**4*s4**(-1)*u2s**(-1)*t2s**(-2) + 384*s*m1**4*
     +    s4**(-1)*t2s**(-3) - 768*s*m1**6*s4**(-2)*u2s**(-1)*t2s**(-2)
     +     + 384*s*m1**6*s4**(-2)*t2s**(-3) - 256*s*m1**6*s4**(-1)*
     +    u2s**(-2)*t2s**(-2) + 768*s*m1**6*s4**(-1)*u2s**(-1)*
     +    t2s**(-3) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac*log_all
     +  * (  - 256*s*m1**8*s4**(-2)*u2s**(-2)*t2s**(-2) + 768*s*m1**8*
     +    s4**(-2)*u2s**(-1)*t2s**(-3) + 384*s*m1**8*s4**(-1)*u2s**(-2)
     +    *t2s**(-3) + 384*s*m1**10*s4**(-2)*u2s**(-2)*t2s**(-3) + 64*s
     +    *s4**(-1)*t2s**(-1) + 128*s**2*(t2*u2+s*m12)**(-2)*m1**6*
     +    u2s**(-1)*t2s**(-1) - 384*s**2*(t2*u2+s*m12)**(-2)*m1**6*
     +    t2s**(-2) + 128*s**2*(t2*u2+s*m12)**(-2)*m1**8*s4**(-1)*
     +    u2s**(-1)*t2s**(-1) - 384*s**2*(t2*u2+s*m12)**(-2)*m1**8*
     +    s4**(-1)*t2s**(-2) - 384*s**2*(t2*u2+s*m12)**(-2)*m1**8*
     +    u2s**(-1)*t2s**(-2) - 384*s**2*(t2*u2+s*m12)**(-2)*m1**10*
     +    s4**(-1)*u2s**(-1)*t2s**(-2) + 128*s**2*(t2*u2+s*m12)**(-1)*
     +    m1**4*u2s**(-1)*t2s**(-2) + 128*s**2*(t2*u2+s*m12)**(-1)*
     +    m1**6*s4**(-1)*u2s**(-1)*t2s**(-2) + 128*s**2*
     +    (t2*u2+s*m12)**(-1)*m1**6*u2s**(-2)*t2s**(-2) - 256*s**2*
     +    (t2*u2+s*m12)**(-1)*m1**6*u2s**(-1)*t2s**(-3) + 128*s**2*
     +    (t2*u2+s*m12)**(-1)*m1**8*s4**(-1)*u2s**(-2)*t2s**(-2) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac*log_all
     +  * (  - 256*s**2*(t2*u2+s*m12)**(-1)*m1**8*s4**(-1)*u2s**(-1)*
     +    t2s**(-3) - 384*s**2*(t2*u2+s*m12)**(-1)*m1**8*u2s**(-2)*
     +    t2s**(-3) - 384*s**2*(t2*u2+s*m12)**(-1)*m1**10*s4**(-1)*
     +    u2s**(-2)*t2s**(-3) - 128*s**2*m1**2*s4**(-1)*u2s**(-1)*
     +    t2s**(-2) - 128*s**2*m1**4*s4**(-2)*u2s**(-1)*t2s**(-2) - 128
     +    *s**2*m1**4*s4**(-1)*u2s**(-2)*t2s**(-2) + 256*s**2*m1**4*
     +    s4**(-1)*u2s**(-1)*t2s**(-3) - 128*s**2*m1**6*s4**(-2)*
     +    u2s**(-2)*t2s**(-2) + 256*s**2*m1**6*s4**(-2)*u2s**(-1)*
     +    t2s**(-3) + 384*s**2*m1**6*s4**(-1)*u2s**(-2)*t2s**(-3) + 384
     +    *s**2*m1**8*s4**(-2)*u2s**(-2)*t2s**(-3) - 128*s**3*
     +    (t2*u2+s*m12)**(-2)*m1**6*u2s**(-1)*t2s**(-2) - 128*s**3*
     +    (t2*u2+s*m12)**(-2)*m1**8*s4**(-1)*u2s**(-1)*t2s**(-2) - 128*
     +    s**3*(t2*u2+s*m12)**(-1)*m1**6*u2s**(-2)*t2s**(-3) - 128*s**3
     +    *(t2*u2+s*m12)**(-1)*m1**8*s4**(-1)*u2s**(-2)*t2s**(-3) + 128
     +    *s**3*m1**4*s4**(-1)*u2s**(-2)*t2s**(-3) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac*log_all
     +  * ( 128*s**3*m1**6*s4**(-2)*u2s**(-2)*t2s**(-3) - 256*
     +    (s+t2)**(-2)*m1**2*s4**(-1)*u2s - 512*(s+t2)**(-2)*m1**2 - 
     +    1024*(s+t2)**(-2)*m1**4*s4**(-1) - 512*(s+t2)**(-2)*m1**6*
     +    s4**(-2) - 256*(s+t2)**(-2)*u2s - 768*(s+t2)**(-1)*m1**2*
     +    s4**(-1) - 1536*(s+t2)**(-1)*m1**4*s4**(-2) - 1024*
     +    (s+t2)**(-1)*m1**6*s4**(-3) - 256*(s+t2)**(-1) + 256*
     +    (s+u2)**(-1)*m1**2*s4**(-1) + 128*(s+u2)**(-1)*m1**4*s4**(-2)
     +     + 128*(s+u2)**(-1) + 128*(t2*u2+s*m12)**(-2)*m1**4*u2 + 128*
     +    (t2*u2+s*m12)**(-2)*m1**6*u2*s4**(-1) + 128*
     +    (t2*u2+s*m12)**(-2)*m1**6*u2*t2s**(-1) - 128*
     +    (t2*u2+s*m12)**(-2)*m1**6*u2**2*t2s**(-2) + 128*
     +    (t2*u2+s*m12)**(-2)*m1**8*u2*s4**(-1)*t2s**(-1) - 128*
     +    (t2*u2+s*m12)**(-2)*m1**8*u2*t2s**(-2) - 128*
     +    (t2*u2+s*m12)**(-2)*m1**8*u2**2*s4**(-1)*t2s**(-2) + 128*
     +    (t2*u2+s*m12)**(-2)*m1**8*t2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac*log_all
     +  * (  - 128*(t2*u2+s*m12)**(-2)*m1**10*u2*s4**(-1)*t2s**(-2) + 
     +    128*(t2*u2+s*m12)**(-2)*m1**10*s4**(-1)*t2s**(-1) + 128*
     +    (t2*u2+s*m12)**(-2)*m1**10*u2s**(-1)*t2s**(-1) - 128*
     +    (t2*u2+s*m12)**(-2)*m1**10*t2s**(-2) + 128*
     +    (t2*u2+s*m12)**(-2)*m1**12*s4**(-1)*u2s**(-1)*t2s**(-1) - 128
     +    *(t2*u2+s*m12)**(-2)*m1**12*s4**(-1)*t2s**(-2) - 128*
     +    (t2*u2+s*m12)**(-2)*m1**12*u2s**(-1)*t2s**(-2) - 128*
     +    (t2*u2+s*m12)**(-2)*m1**14*s4**(-1)*u2s**(-1)*t2s**(-2) - 256
     +    *(t2*u2+s*m12)**(-1)*m1**2 + 256*(t2*u2+s*m12)**(-1)*m1**4*u2
     +    *t2s**(-2) - 256*(t2*u2+s*m12)**(-1)*m1**4*s4**(-1) - 128*
     +    (t2*u2+s*m12)**(-1)*m1**4*u2s**(-1) - 256*(t2*u2+s*m12)**(-1)
     +    *m1**4*t2s**(-1) + 256*(t2*u2+s*m12)**(-1)*m1**6*u2*s4**(-1)*
     +    t2s**(-2) - 384*(t2*u2+s*m12)**(-1)*m1**6*u2*t2s**(-3) - 128*
     +    (t2*u2+s*m12)**(-1)*m1**6*s4**(-1)*u2s**(-1) - 256*
     +    (t2*u2+s*m12)**(-1)*m1**6*s4**(-1)*t2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac*log_all
     +  * (  - 384*(t2*u2+s*m12)**(-1)*m1**6*u2s**(-1)*t2s**(-1) + 384*
     +    (t2*u2+s*m12)**(-1)*m1**6*t2s**(-2) + 128*(t2*u2+s*m12)**(-1)
     +    *m1**8*t2**(-1)*u2*t2s**(-3) + 128*(t2*u2+s*m12)**(-1)*m1**8*
     +    t2**(-1)*u2s**(-1)*t2s**(-1) - 384*(t2*u2+s*m12)**(-1)*m1**8*
     +    u2*s4**(-1)*t2s**(-3) - 384*(t2*u2+s*m12)**(-1)*m1**8*
     +    s4**(-1)*u2s**(-1)*t2s**(-1) + 384*(t2*u2+s*m12)**(-1)*m1**8*
     +    s4**(-1)*t2s**(-2) + 512*(t2*u2+s*m12)**(-1)*m1**8*u2s**(-1)*
     +    t2s**(-2) - 512*(t2*u2+s*m12)**(-1)*m1**8*t2s**(-3) + 128*
     +    (t2*u2+s*m12)**(-1)*m1**10*t2**(-1)*u2*s4**(-1)*t2s**(-3) + 
     +    128*(t2*u2+s*m12)**(-1)*m1**10*t2**(-1)*s4**(-1)*u2s**(-1)*
     +    t2s**(-1) + 128*(t2*u2+s*m12)**(-1)*m1**10*t2**(-1)*t2s**(-3)
     +     + 512*(t2*u2+s*m12)**(-1)*m1**10*s4**(-1)*u2s**(-1)*
     +    t2s**(-2) - 512*(t2*u2+s*m12)**(-1)*m1**10*s4**(-1)*t2s**(-3)
     +     + 128*(t2*u2+s*m12)**(-1)*m1**10*u2s**(-2)*t2s**(-2) - 640*
     +    (t2*u2+s*m12)**(-1)*m1**10*u2s**(-1)*t2s**(-3) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac*log_all
     +  * ( 128*(t2*u2+s*m12)**(-1)*m1**12*t2**(-1)*s4**(-1)*t2s**(-3)
     +     + 128*(t2*u2+s*m12)**(-1)*m1**12*t2**(-1)*u2s**(-1)*
     +    t2s**(-3) + 128*(t2*u2+s*m12)**(-1)*m1**12*s4**(-1)*u2s**(-2)
     +    *t2s**(-2) - 640*(t2*u2+s*m12)**(-1)*m1**12*s4**(-1)*
     +    u2s**(-1)*t2s**(-3) - 128*(t2*u2+s*m12)**(-1)*m1**12*
     +    u2s**(-2)*t2s**(-3) + 128*(t2*u2+s*m12)**(-1)*m1**14*t2**(-1)
     +    *s4**(-1)*u2s**(-1)*t2s**(-3) - 128*(t2*u2+s*m12)**(-1)*
     +    m1**14*s4**(-1)*u2s**(-2)*t2s**(-3) + 128*m1**2*t2*u2*
     +    s4**(-2)*t2s**(-2) - 256*m1**2*t2*s4**(-2)*u2s**(-1) - 256*
     +    m1**2*t2*s4**(-1)*u2s**(-2) + 512*m1**2*t2*u2s**(-3) - 384*
     +    m1**2*u2*s4**(-1)*t2s**(-2) - 128*m1**2*s4**(-2) - 896*m1**2*
     +    s4**(-1)*u2s**(-1) + 256*m1**2*s4**(-1)*t2s**(-1) - 512*m1**4
     +    *t2*s4**(-2)*u2s**(-2) + 512*m1**4*t2*u2s**(-4) - 384*m1**4*
     +    u2*s4**(-2)*t2s**(-2) + 384*m1**4*u2*s4**(-1)*t2s**(-3) - 
     +    1920*m1**4*s4**(-2)*u2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac*log_all
     +  * ( 256*m1**4*s4**(-2)*t2s**(-1) - 512*m1**4*s4**(-1)*u2s**(-2)
     +     + 384*m1**4*s4**(-1)*u2s**(-1)*t2s**(-1) - 384*m1**4*
     +    s4**(-1)*t2s**(-2) - 512*m1**6*t2*s4**(-2)*u2s**(-3) + 512*
     +    m1**6*t2*s4**(-1)*u2s**(-4) + 384*m1**6*u2*s4**(-2)*t2s**(-3)
     +     - 1024*m1**6*s4**(-3)*u2s**(-1) - 512*m1**6*s4**(-2)*
     +    u2s**(-2) + 384*m1**6*s4**(-2)*u2s**(-1)*t2s**(-1) - 384*
     +    m1**6*s4**(-2)*t2s**(-2) - 512*m1**6*s4**(-1)*u2s**(-1)*
     +    t2s**(-2) + 512*m1**6*s4**(-1)*t2s**(-3) - 512*m1**8*s4**(-2)
     +    *u2s**(-1)*t2s**(-2) + 512*m1**8*s4**(-2)*t2s**(-3) - 128*
     +    m1**8*s4**(-1)*u2s**(-2)*t2s**(-2) + 640*m1**8*s4**(-1)*
     +    u2s**(-1)*t2s**(-3) - 128*m1**10*s4**(-2)*u2s**(-2)*t2s**(-2)
     +     + 640*m1**10*s4**(-2)*u2s**(-1)*t2s**(-3) + 128*m1**10*
     +    s4**(-1)*u2s**(-2)*t2s**(-3) + 128*m1**12*s4**(-2)*u2s**(-2)*
     +    t2s**(-3) + 128*t2*u2*s4**(-1)*t2s**(-2) - 256*t2*s4**(-1)*
     +    u2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac*log_all
     +  * ( 256*t2*u2s**(-2) - 128*s4**(-1) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac * (  - 
     +    128*s**(-1)*(s+t2)**(-1)*m1**2*u2*s4**(-1) - 128*s**(-1)*
     +    (s+t2)**(-1)*m1**2*u2**2*s4**(-2) + 512*s**(-1)*(s+t2)**(-1)*
     +    m1**2*s4**(-1)*u2s + 384*s**(-1)*(s+t2)**(-1)*m1**2 + 256*
     +    s**(-1)*(s+t2)**(-1)*m1**4*u2*s4**(-2) + 256*s**(-1)*
     +    (s+t2)**(-1)*m1**4*s4**(-1) - 128*s**(-1)*(s+t2)**(-1)*m1**6*
     +    s4**(-2) - 384*s**(-1)*(s+t2)**(-1)*u2 - 128*s**(-1)*
     +    (s+t2)**(-1)*u2**2*s4**(-1) + 512*s**(-1)*(s+t2)**(-1)*u2s - 
     +    64*s**(-1)*m1**2*t2**(-1)*u2*s4**(-1) + 64*s**(-1)*m1**2*
     +    t2**(-1)*u2*t2s**(-1) + 64*s**(-1)*m1**2*t2**(-1) - 160*
     +    s**(-1)*m1**2*t2*s4**(-2) + 192*s**(-1)*m1**2*t2*s4**(-1)*
     +    u2s**(-1) - 512*s**(-1)*m1**2*t2*u2s**(-2) - 192*s**(-1)*
     +    m1**2*t2**2*s4**(-2)*u2s**(-1) - 512*s**(-1)*m1**2*t2**2*
     +    u2s**(-3) - 32*s**(-1)*m1**2*u2*s4**(-2) - 64*s**(-1)*m1**2*
     +    u2*s4**(-1)*t2s**(-1) - 128*s**(-1)*m1**2*u2*t2s**(-2) + 64*
     +    s**(-1)*m1**2*u2**2*s4**(-2)*t2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac * ( 64*
     +    s**(-1)*m1**2*u2**2*s4**(-1)*t2s**(-2) - 64*s**(-1)*m1**2*
     +    s4**(-1) + 64*s**(-1)*m1**2*t2s**(-1) + 64*s**(-1)*m1**4*
     +    t2**(-1)*u2*s4**(-1)*t2s**(-1) + 64*s**(-1)*m1**4*t2**(-1)*
     +    s4**(-1) - 64*s**(-1)*m1**4*t2**(-1)*t2s**(-1) - 512*s**(-1)*
     +    m1**4*t2*s4**(-1)*u2s**(-2) - 512*s**(-1)*m1**4*t2*u2s**(-3)
     +     - 512*s**(-1)*m1**4*t2**2*s4**(-1)*u2s**(-3) - 512*s**(-1)*
     +    m1**4*t2**2*u2s**(-4) - 128*s**(-1)*m1**4*u2*s4**(-1)*
     +    t2s**(-2) + 64*s**(-1)*m1**4*u2**2*s4**(-2)*t2s**(-2) + 32*
     +    s**(-1)*m1**4*s4**(-2) + 64*s**(-1)*m1**4*s4**(-1)*t2s**(-1)
     +     - 64*s**(-1)*m1**6*t2**(-1)*s4**(-1)*t2s**(-1) - 512*s**(-1)
     +    *m1**6*t2*s4**(-1)*u2s**(-3) - 512*s**(-1)*m1**6*t2**2*
     +    s4**(-1)*u2s**(-4) - 64*s**(-1)*t2**(-1)*u2 - 160*s**(-1)*t2*
     +    s4**(-1) + 192*s**(-1)*t2*u2s**(-1) - 192*s**(-1)*t2**2*
     +    s4**(-1)*u2s**(-1) - 32*s**(-1)*u2*s4**(-1) - 64*s**(-1)*u2*
     +    t2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac * ( 64*
     +    s**(-1)*u2**2*s4**(-1)*t2s**(-1) - 96*s**(-1) + 896*s*
     +    (t2*u2+s*m12)**(-2)*m1**2*t2 - 128*s*(t2*u2+s*m12)**(-2)*
     +    m1**2*t2**2*s4**(-1) + 2304*s*(t2*u2+s*m12)**(-2)*m1**2*u2 - 
     +    640*s*(t2*u2+s*m12)**(-2)*m1**2*u2**2*s4**(-1) - 896*s*
     +    (t2*u2+s*m12)**(-2)*m1**2*s4 + 640*s*(t2*u2+s*m12)**(-2)*
     +    m1**4*t2*s4**(-1) + 1664*s*(t2*u2+s*m12)**(-2)*m1**4*u2*
     +    s4**(-1) - 1792*s*(t2*u2+s*m12)**(-2)*m1**4 - 1024*s*
     +    (t2*u2+s*m12)**(-2)*m1**6*s4**(-1) + 256*s*
     +    (t2*u2+s*m12)**(-2)*t2*s4 - 128*s*(t2*u2+s*m12)**(-2)*t2**2
     +     + 640*s*(t2*u2+s*m12)**(-2)*u2*s4 - 640*s*
     +    (t2*u2+s*m12)**(-2)*u2**2 - 128*s*(t2*u2+s*m12)**(-2)*s4**2
     +     - 64*s*(t2*u2+s*m12)**(-1)*m1**2*t2**(-1) - 832*s*
     +    (t2*u2+s*m12)**(-1)*m1**2*s4**(-1) - 64*s*(t2*u2+s*m12)**(-1)
     +    *m1**2*u2s**(-1) - 64*s*(t2*u2+s*m12)**(-1)*m1**4*t2**(-1)*
     +    s4**(-1) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac * (  - 64
     +    *s*(t2*u2+s*m12)**(-1)*m1**4*s4**(-1)*u2s**(-1) - 832*s*
     +    (t2*u2+s*m12)**(-1) + 64*s*m1**2*t2**(-1)*s4**(-1)*u2s**(-1)
     +     + 256*s*m1**2*s4**(-2)*u2s**(-1) + 64*s*m1**2*s4**(-2)*
     +    t2s**(-1) + 256*s*m1**2*s4**(-1)*u2s**(-2) + 64*s*m1**2*
     +    s4**(-1)*t2s**(-2) + 256*s*m1**4*s4**(-2)*u2s**(-2) + 64*s*
     +    m1**4*s4**(-2)*t2s**(-2) + 64*s*t2**(-1)*u2s**(-1) + 256*s*
     +    s4**(-1)*u2s**(-1) + 64*s*s4**(-1)*t2s**(-1) - 256*s**2*
     +    (t2*u2+s*m12)**(-2)*m1**2*t2*s4**(-1) - 512*s**2*
     +    (t2*u2+s*m12)**(-2)*m1**2*u2*s4**(-1) + 1408*s**2*
     +    (t2*u2+s*m12)**(-2)*m1**2 + 1152*s**2*(t2*u2+s*m12)**(-2)*
     +    m1**4*s4**(-1) - 256*s**2*(t2*u2+s*m12)**(-2)*t2 - 512*s**2*
     +    (t2*u2+s*m12)**(-2)*u2 + 256*s**2*(t2*u2+s*m12)**(-2)*s4 - 64
     +    *s**2*(t2*u2+s*m12)**(-1)*m1**2*t2**(-1)*u2s**(-1) - 64*s**2*
     +    (t2*u2+s*m12)**(-1)*m1**4*t2**(-1)*s4**(-1)*u2s**(-1) - 128*
     +    s**3*(t2*u2+s*m12)**(-2)*m1**2*s4**(-1) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac * (  - 
     +    128*s**3*(t2*u2+s*m12)**(-2) + 256*(s+t2)**(-2)*m1**2*u2*
     +    s4**(-1) - 256*(s+t2)**(-2)*m1**2*s4**(-1)*u2s - 256*
     +    (s+t2)**(-2)*m1**2 - 256*(s+t2)**(-2)*m1**4*s4**(-1) + 256*
     +    (s+t2)**(-2)*u2 - 256*(s+t2)**(-2)*u2s - 320*(s+t2)**(-1)*
     +    m1**2*u2*s4**(-2) + 128*(s+t2)**(-1)*m1**2*u2**2*s4**(-3) + 
     +    128*(s+t2)**(-1)*m1**2*s4**(-1) - 128*(s+t2)**(-1)*m1**4*u2*
     +    s4**(-3) + 64*(s+t2)**(-1)*m1**4*s4**(-2) - 192*(s+t2)**(-1)*
     +    u2*s4**(-1) + 128*(s+t2)**(-1)*u2**2*s4**(-2) + 64*
     +    (s+t2)**(-1) - 640*(t2*u2+s*m12)**(-2)*m1**2*u2*s4 + 1152*
     +    (t2*u2+s*m12)**(-2)*m1**2*u2**2 - 256*(t2*u2+s*m12)**(-2)*
     +    m1**2*u2**3*s4**(-1) - 1024*(t2*u2+s*m12)**(-2)*m1**4*u2 + 
     +    768*(t2*u2+s*m12)**(-2)*m1**4*u2**2*s4**(-1) - 512*
     +    (t2*u2+s*m12)**(-2)*m1**6*u2*s4**(-1) - 128*
     +    (t2*u2+s*m12)**(-2)*u2*s4**2 + 384*(t2*u2+s*m12)**(-2)*u2**2*
     +    s4 )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac * (  - 
     +    256*(t2*u2+s*m12)**(-2)*u2**3 + 128*(t2*u2+s*m12)**(-1)*m1**2
     +    *t2**(-1)*u2 - 256*(t2*u2+s*m12)**(-1)*m1**2*t2*s4**(-1) - 
     +    576*(t2*u2+s*m12)**(-1)*m1**2*u2*s4**(-1) + 1152*
     +    (t2*u2+s*m12)**(-1)*m1**2 + 128*(t2*u2+s*m12)**(-1)*m1**4*
     +    t2**(-1)*u2*s4**(-1) - 128*(t2*u2+s*m12)**(-1)*m1**4*t2**(-1)
     +     + 768*(t2*u2+s*m12)**(-1)*m1**4*s4**(-1) - 128*
     +    (t2*u2+s*m12)**(-1)*m1**6*t2**(-1)*s4**(-1) - 256*
     +    (t2*u2+s*m12)**(-1)*t2 - 576*(t2*u2+s*m12)**(-1)*u2 + 384*
     +    (t2*u2+s*m12)**(-1)*s4 - 64*m1**2*t2**(-1)*t2s**(-1) + 64*
     +    m1**2*t2*s4**(-2)*u2s**(-1) + 256*m1**2*t2*s4**(-1)*u2s**(-2)
     +     - 512*m1**2*t2*u2s**(-3) + 128*m1**2*u2*s4**(-3) + 128*m1**2
     +    *u2*s4**(-2)*t2s**(-1) + 128*m1**2*u2*s4**(-1)*t2s**(-2) + 
     +    288*m1**2*s4**(-2) - 64*m1**2*s4**(-1)*t2s**(-1) - 256*m1**2*
     +    u2s**(-2) - 128*m1**2*t2s**(-2) - 64*m1**4*t2**(-1)*s4**(-1)*
     +    t2s**(-1) )
      MM_qg = MM_qg + Ar2*Nc**3*st**2*Pi**3*alphas**2*hardfac * ( 256*
     +    m1**4*t2*s4**(-2)*u2s**(-2) - 512*m1**4*t2*s4**(-1)*u2s**(-3)
     +     - 512*m1**4*t2*u2s**(-4) + 128*m1**4*u2*s4**(-2)*t2s**(-2)
     +     + 256*m1**4*s4**(-2)*u2s**(-1) - 256*m1**4*s4**(-1)*
     +    u2s**(-2) - 128*m1**4*s4**(-1)*t2s**(-2) - 512*m1**6*t2*
     +    s4**(-1)*u2s**(-4) + 64*t2*s4**(-1)*u2s**(-1) + 128*u2*
     +    s4**(-2) + 128*u2*s4**(-1)*t2s**(-1) + 288*s4**(-1) - 256*
     +    u2s**(-1) - 64*t2s**(-1) )
      MM_qg = MM_qg + ANG_fin(0,0,0,0)*Ar2*Nc**(-1)*st**2*Pi**2*
     + alphas**2*hardfac * (  - 256*s**(-1)*m1**2*t2*s4**(-2) + 160*
     +    s**(-1)*m1**2*t2*s4**(-1)*u2s**(-1) - 128*s**(-1)*m1**2*t2**2
     +    *s4**(-2)*u2s**(-1) - 128*s**(-1)*m1**2*u2*s4**(-2) + 192*
     +    s**(-1)*m1**2*s4**(-1) - 64*s**(-1)*m1**2*u2s**(-1) - 256*
     +    s**(-1)*m1**4*t2*s4**(-2)*u2s**(-1) - 128*s**(-1)*m1**4*
     +    s4**(-2) + 192*s**(-1)*m1**4*s4**(-1)*u2s**(-1) - 128*s**(-1)
     +    *m1**6*s4**(-2)*u2s**(-1) - 192*s**(-1)*t2*u2*s4**(-2) + 96*
     +    s**(-1)*t2*s4**(-1) - 64*s**(-1)*t2**2*s4**(-2) + 160*s**(-1)
     +    *u2*s4**(-1) - 128*s**(-1)*u2**2*s4**(-2) - 32*s**(-1) - 256*
     +    s*m1**2*s4**(-2)*u2s**(-1) - 128*s*m1**4*s4**(-2)*u2s**(-2)
     +     - 128*s*s4**(-2) + 64*s*s4**(-1)*u2s**(-1) - 384*m1**2*t2*
     +    s4**(-2)*u2s**(-1) - 384*m1**2*s4**(-2) + 256*m1**2*s4**(-1)*
     +    u2s**(-1) - 64*m1**2*u2s**(-2) - 128*m1**4*t2*s4**(-2)*
     +    u2s**(-2) - 512*m1**4*s4**(-2)*u2s**(-1) - 128*m1**6*s4**(-2)
     +    *u2s**(-2) )
      MM_qg = MM_qg + ANG_fin(0,0,0,0)*Ar2*Nc**(-1)*st**2*Pi**2*
     + alphas**2*hardfac * (  - 192*t2*s4**(-2) + 32*t2*s4**(-1)*
     +    u2s**(-1) - 256*u2*s4**(-2) + 224*s4**(-1) - 64*u2s**(-1) )
      MM_qg = MM_qg + ANG_fin(0,0,0,0)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 512*s**(-1)*m1**2*t2*s4**(-2) - 320*s**(-1)*m1**2*t2
     +    *s4**(-1)*u2s**(-1) + 256*s**(-1)*m1**2*t2**2*s4**(-2)*
     +    u2s**(-1) + 256*s**(-1)*m1**2*u2*s4**(-2) - 384*s**(-1)*m1**2
     +    *s4**(-1) + 128*s**(-1)*m1**2*u2s**(-1) + 512*s**(-1)*m1**4*
     +    t2*s4**(-2)*u2s**(-1) + 256*s**(-1)*m1**4*s4**(-2) - 384*
     +    s**(-1)*m1**4*s4**(-1)*u2s**(-1) + 256*s**(-1)*m1**6*s4**(-2)
     +    *u2s**(-1) + 384*s**(-1)*t2*u2*s4**(-2) - 192*s**(-1)*t2*
     +    s4**(-1) + 128*s**(-1)*t2**2*s4**(-2) - 320*s**(-1)*u2*
     +    s4**(-1) + 256*s**(-1)*u2**2*s4**(-2) + 64*s**(-1) + 512*s*
     +    m1**2*s4**(-2)*u2s**(-1) + 256*s*m1**4*s4**(-2)*u2s**(-2) + 
     +    256*s*s4**(-2) - 128*s*s4**(-1)*u2s**(-1) + 768*m1**2*t2*
     +    s4**(-2)*u2s**(-1) + 768*m1**2*s4**(-2) - 512*m1**2*s4**(-1)*
     +    u2s**(-1) + 128*m1**2*u2s**(-2) + 256*m1**4*t2*s4**(-2)*
     +    u2s**(-2) + 1024*m1**4*s4**(-2)*u2s**(-1) + 256*m1**6*
     +    s4**(-2)*u2s**(-2) )
      MM_qg = MM_qg + ANG_fin(0,0,0,0)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 384*t2*s4**(-2) - 64*t2*s4**(-1)*u2s**(-1) + 512*u2*
     +    s4**(-2) - 448*s4**(-1) + 128*u2s**(-1) )
      MM_qg = MM_qg + ANG_fin(0,0,0,0)*Ar2*Nc**3*st**2*Pi**2*alphas**2*
     + hardfac * (  - 256*s**(-1)*m1**2*t2*s4**(-2) + 160*s**(-1)*m1**2
     +    *t2*s4**(-1)*u2s**(-1) - 128*s**(-1)*m1**2*t2**2*s4**(-2)*
     +    u2s**(-1) - 128*s**(-1)*m1**2*u2*s4**(-2) + 192*s**(-1)*m1**2
     +    *s4**(-1) - 64*s**(-1)*m1**2*u2s**(-1) - 256*s**(-1)*m1**4*t2
     +    *s4**(-2)*u2s**(-1) - 128*s**(-1)*m1**4*s4**(-2) + 192*
     +    s**(-1)*m1**4*s4**(-1)*u2s**(-1) - 128*s**(-1)*m1**6*s4**(-2)
     +    *u2s**(-1) - 192*s**(-1)*t2*u2*s4**(-2) + 96*s**(-1)*t2*
     +    s4**(-1) - 64*s**(-1)*t2**2*s4**(-2) + 160*s**(-1)*u2*
     +    s4**(-1) - 128*s**(-1)*u2**2*s4**(-2) - 32*s**(-1) - 256*s*
     +    m1**2*s4**(-2)*u2s**(-1) - 128*s*m1**4*s4**(-2)*u2s**(-2) - 
     +    128*s*s4**(-2) + 64*s*s4**(-1)*u2s**(-1) - 384*m1**2*t2*
     +    s4**(-2)*u2s**(-1) - 384*m1**2*s4**(-2) + 256*m1**2*s4**(-1)*
     +    u2s**(-1) - 64*m1**2*u2s**(-2) - 128*m1**4*t2*s4**(-2)*
     +    u2s**(-2) - 512*m1**4*s4**(-2)*u2s**(-1) - 128*m1**6*s4**(-2)
     +    *u2s**(-2) )
      MM_qg = MM_qg + ANG_fin(0,0,0,0)*Ar2*Nc**3*st**2*Pi**2*alphas**2*
     + hardfac * (  - 192*t2*s4**(-2) + 32*t2*s4**(-1)*u2s**(-1) - 256*
     +    u2*s4**(-2) + 224*s4**(-1) - 64*u2s**(-1) )
      MM_qg = MM_qg + ANG_fin(1,0,-2,0)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 256*s*m1**2*t2*u2s**(-2) + 128*s**2*m1**2*u2s**(-2)
     +     + 128*m1**2*t2**2*u2s**(-2) )
      MM_qg = MM_qg + ANG_fin(1,0,-2,0)*Ar2*Nc**3*st**2*Pi**2*alphas**2
     + *hardfac * (  - 256*s*m1**2*t2*u2s**(-2) - 128*s**2*m1**2*
     +    u2s**(-2) - 128*m1**2*t2**2*u2s**(-2) )
      MM_qg = MM_qg + ANG_fin(1,0,-1,0)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * (  - 32 - 288*s**(-1)*m1**2*t2*s4**(-1) + 192*s**(-1)*
     +    m1**2*t2*u2s**(-1) - 224*s**(-1)*m1**2*t2**2*s4**(-1)*
     +    u2s**(-1) - 64*s**(-1)*m1**2*u2*s4**(-1) + 320*s**(-1)*m1**2
     +     - 128*s**(-1)*m1**4*t2*s4**(-1)*u2s**(-1) + 128*s**(-1)*
     +    m1**4*s4**(-1) + 128*s**(-1)*m1**4*u2s**(-1) + 128*s**(-1)*
     +    m1**6*s4**(-1)*u2s**(-1) - 160*s**(-1)*t2*u2*s4**(-1) - 64*
     +    s**(-1)*t2 - 160*s**(-1)*t2**2*s4**(-1) + 32*s**(-1)*t2**2*
     +    u2s**(-1) - 64*s**(-1)*t2**3*s4**(-1)*u2s**(-1) - 64*s**(-1)*
     +    u2 - 64*s**(-1)*u2**2*s4**(-1) + 128*s**(-1)*s4 - 32*s*
     +    (s+t2)**(-1)*m1**2*s4**(-1) + 32*s*(s+t2)**(-1)*u2*s4**(-1)
     +     - 32*s*(s+t2)**(-1) + 64*s*m1**2*s4**(-1)*u2s**(-1) + 128*s*
     +    m1**2*u2s**(-2) + 128*s*m1**4*s4**(-1)*u2s**(-2) - 224*s*t2*
     +    s4**(-1)*u2s**(-1) - 64*s*s4**(-1) + 160*s*u2s**(-1) - 96*
     +    s**2*s4**(-1)*u2s**(-1) + 192*(s+t2)**(-1)*m1**2*u2*s4**(-1)
     +     - 128*(s+t2)**(-1)*m1**2*u2**2*s4**(-2) )
      MM_qg = MM_qg + ANG_fin(1,0,-1,0)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * (  - 64*(s+t2)**(-1)*m1**2 + 32*(s+t2)**(-1)*m1**4*u2*
     +    s4**(-2) + 96*(s+t2)**(-1)*u2 - 128*(s+t2)**(-1)*u2**2*
     +    s4**(-1) + 96*(s+t2)**(-1)*u2**3*s4**(-2) - 160*m1**2*t2*
     +    s4**(-1)*u2s**(-1) + 128*m1**2*t2*u2s**(-2) - 32*m1**2*u2*
     +    s4**(-2) + 32*m1**2*s4**(-1) + 224*m1**2*u2s**(-1) + 128*
     +    m1**4*t2*s4**(-1)*u2s**(-2) + 256*m1**4*s4**(-1)*u2s**(-1) + 
     +    128*m1**4*u2s**(-2) - 224*t2*s4**(-1) + 160*t2*u2s**(-1) - 
     +    192*t2**2*s4**(-1)*u2s**(-1) - 96*u2*s4**(-1) + 96*u2**2*
     +    s4**(-2) )
      MM_qg = MM_qg + ANG_fin(1,0,-1,0)*Ar2*Nc**3*st**2*Pi**2*alphas**2
     + *hardfac * ( 32 + 288*s**(-1)*m1**2*t2*s4**(-1) - 192*s**(-1)*
     +    m1**2*t2*u2s**(-1) + 224*s**(-1)*m1**2*t2**2*s4**(-1)*
     +    u2s**(-1) + 64*s**(-1)*m1**2*u2*s4**(-1) - 320*s**(-1)*m1**2
     +     + 128*s**(-1)*m1**4*t2*s4**(-1)*u2s**(-1) - 128*s**(-1)*
     +    m1**4*s4**(-1) - 128*s**(-1)*m1**4*u2s**(-1) - 128*s**(-1)*
     +    m1**6*s4**(-1)*u2s**(-1) + 160*s**(-1)*t2*u2*s4**(-1) + 64*
     +    s**(-1)*t2 + 160*s**(-1)*t2**2*s4**(-1) - 32*s**(-1)*t2**2*
     +    u2s**(-1) + 64*s**(-1)*t2**3*s4**(-1)*u2s**(-1) + 64*s**(-1)*
     +    u2 + 64*s**(-1)*u2**2*s4**(-1) - 128*s**(-1)*s4 + 32*s*
     +    (s+t2)**(-1)*m1**2*s4**(-1) - 32*s*(s+t2)**(-1)*u2*s4**(-1)
     +     + 32*s*(s+t2)**(-1) - 64*s*m1**2*s4**(-1)*u2s**(-1) - 128*s*
     +    m1**2*u2s**(-2) - 128*s*m1**4*s4**(-1)*u2s**(-2) + 224*s*t2*
     +    s4**(-1)*u2s**(-1) + 64*s*s4**(-1) - 160*s*u2s**(-1) + 96*
     +    s**2*s4**(-1)*u2s**(-1) - 192*(s+t2)**(-1)*m1**2*u2*s4**(-1)
     +     + 128*(s+t2)**(-1)*m1**2*u2**2*s4**(-2) )
      MM_qg = MM_qg + ANG_fin(1,0,-1,0)*Ar2*Nc**3*st**2*Pi**2*alphas**2
     + *hardfac * ( 64*(s+t2)**(-1)*m1**2 - 32*(s+t2)**(-1)*m1**4*u2*
     +    s4**(-2) - 96*(s+t2)**(-1)*u2 + 128*(s+t2)**(-1)*u2**2*
     +    s4**(-1) - 96*(s+t2)**(-1)*u2**3*s4**(-2) + 160*m1**2*t2*
     +    s4**(-1)*u2s**(-1) - 128*m1**2*t2*u2s**(-2) + 32*m1**2*u2*
     +    s4**(-2) - 32*m1**2*s4**(-1) - 224*m1**2*u2s**(-1) - 128*
     +    m1**4*t2*s4**(-1)*u2s**(-2) - 256*m1**4*s4**(-1)*u2s**(-1) - 
     +    128*m1**4*u2s**(-2) + 224*t2*s4**(-1) - 160*t2*u2s**(-1) + 
     +    192*t2**2*s4**(-1)*u2s**(-1) + 96*u2*s4**(-1) - 96*u2**2*
     +    s4**(-2) )
      MM_qg = MM_qg + ANG_fin(1,2,-1,-1)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 32*s*(s+t2)**(-1)*m1**2 + 128*s*(s+t2)**(-1)*m1**4*
     +    s4**(-1) + 32*s*(s+t2)**(-1)*u2 + 64*s*m1**2*t2*s4**(-1)*
     +    u2s**(-1) + 96*s*m1**2*s4**(-1) - 96*s*m1**2*u2s**(-1) + 96*s
     +    *m1**4*s4**(-1)*u2s**(-1) - 32*s*t2*s4**(-1) - 64*s*t2*
     +    u2s**(-1) + 32*s*t2**2*s4**(-1)*u2s**(-1) + 64*s*u2*s4**(-1)
     +     + 32*s + 96*s**2*m1**2*s4**(-1)*u2s**(-1) + 64*s**2*t2*
     +    s4**(-1)*u2s**(-1) - 32*s**2*u2s**(-1) + 32*s**3*s4**(-1)*
     +    u2s**(-1) - 160*(s+t2)**(-1)*m1**2*u2 + 192*(s+t2)**(-1)*
     +    m1**2*u2**2*s4**(-1) - 128*(s+t2)**(-1)*m1**4*u2*s4**(-1) + 
     +    32*(s+t2)**(-1)*m1**4 - 32*(s+t2)**(-1)*u2*u2s + 128*
     +    (s+t2)**(-1)*u2**2 - 64*(s+t2)**(-1)*u2**3*s4**(-1) - 32*
     +    m1**2*t2*s4**(-1) + 96*m1**2*t2*u2s**(-1) - 32*m1**2*t2**2*
     +    s4**(-1)*u2s**(-1) + 32*m1**2 - 32*m1**4*t2*s4**(-1)*
     +    u2s**(-1) - 32*m1**4*u2s**(-1) + 64*t2*u2*s4**(-1) + 32*t2 - 
     +    32*t2**2*s4**(-1) )
      MM_qg = MM_qg + ANG_fin(1,2,-1,-1)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * (  - 32*t2**2*u2s**(-1) - 96*u2 + 32*u2**2*s4**(-1) )
      MM_qg = MM_qg + ANG_fin(1,2,-1,-1)*Ar2*Nc**3*st**2*Pi**2*
     + alphas**2*hardfac * (  - 32*s*(s+t2)**(-1)*m1**2 - 128*s*
     +    (s+t2)**(-1)*m1**4*s4**(-1) - 32*s*(s+t2)**(-1)*u2 - 64*s*
     +    m1**2*t2*s4**(-1)*u2s**(-1) - 96*s*m1**2*s4**(-1) + 96*s*
     +    m1**2*u2s**(-1) - 96*s*m1**4*s4**(-1)*u2s**(-1) + 32*s*t2*
     +    s4**(-1) + 64*s*t2*u2s**(-1) - 32*s*t2**2*s4**(-1)*u2s**(-1)
     +     - 64*s*u2*s4**(-1) - 32*s - 96*s**2*m1**2*s4**(-1)*u2s**(-1)
     +     - 64*s**2*t2*s4**(-1)*u2s**(-1) + 32*s**2*u2s**(-1) - 32*
     +    s**3*s4**(-1)*u2s**(-1) + 160*(s+t2)**(-1)*m1**2*u2 - 192*
     +    (s+t2)**(-1)*m1**2*u2**2*s4**(-1) + 128*(s+t2)**(-1)*m1**4*u2
     +    *s4**(-1) - 32*(s+t2)**(-1)*m1**4 + 32*(s+t2)**(-1)*u2*u2s - 
     +    128*(s+t2)**(-1)*u2**2 + 64*(s+t2)**(-1)*u2**3*s4**(-1) + 32*
     +    m1**2*t2*s4**(-1) - 96*m1**2*t2*u2s**(-1) + 32*m1**2*t2**2*
     +    s4**(-1)*u2s**(-1) - 32*m1**2 + 32*m1**4*t2*s4**(-1)*
     +    u2s**(-1) + 32*m1**4*u2s**(-1) - 64*t2*u2*s4**(-1) - 32*t2 + 
     +    32*t2**2*s4**(-1) )
      MM_qg = MM_qg + ANG_fin(1,2,-1,-1)*Ar2*Nc**3*st**2*Pi**2*
     + alphas**2*hardfac * ( 32*t2**2*u2s**(-1) + 96*u2 - 32*u2**2*
     +    s4**(-1) )
      MM_qg = MM_qg + ANG_fin(1,5,-2,-2)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 128*s**2*m1**2 )
      MM_qg = MM_qg + ANG_fin(1,5,-2,-2)*Ar2*Nc**3*st**2*Pi**2*
     + alphas**2*hardfac * (  - 128*s**2*m1**2 )
      MM_qg = MM_qg + ANG_fin(1,5,-2,-1)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 256*s*m1**2*t2*u2s**(-1) + 256*s**2*m1**2*u2s**(-1)
     +     )
      MM_qg = MM_qg + ANG_fin(1,5,-2,-1)*Ar2*Nc**3*st**2*Pi**2*
     + alphas**2*hardfac * (  - 256*s*m1**2*t2*u2s**(-1) - 256*s**2*
     +    m1**2*u2s**(-1) )
      MM_qg = MM_qg + ANG_fin(1,5,-1,-2)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 128*s*m1**2 )
      MM_qg = MM_qg + ANG_fin(1,5,-1,-2)*Ar2*Nc**3*st**2*Pi**2*
     + alphas**2*hardfac * (  - 128*s*m1**2 )
      MM_qg = MM_qg + ANG_fin(1,5,-1,-1)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * (  - 64*s*(s+t2)**(-1)*m1**2*u2*s4**(-1) + 64*s*
     +    (s+t2)**(-1)*m1**2 - 32*s*(s+t2)**(-1)*m1**4*s4**(-1) - 32*s*
     +    (s+t2)**(-1)*u2**2*s4**(-1) - 64*s*m1**2*s4**(-1) + 256*s*
     +    m1**2*u2s**(-1) - 128*s*m1**4*s4**(-1)*u2s**(-1) + 128*s*t2*
     +    u2s**(-1) - 64*s*u2*s4**(-1) + 64*s - 64*(s+t2)**(-1)*m1**2*
     +    u2 + 64*(s+t2)**(-1)*m1**2*u2**2*s4**(-1) - 64*(s+t2)**(-1)*
     +    m1**4*u2*s4**(-1) + 32*(s+t2)**(-1)*m1**4 + 32*(s+t2)**(-1)*
     +    u2**2 + 64*m1**2*t2*s4**(-1) + 128*m1**2*t2*u2s**(-1) + 128*
     +    m1**2*u2*s4**(-1) + 160*m1**2 + 128*m1**4*t2*s4**(-1)*
     +    u2s**(-1) + 32*m1**4*s4**(-1) + 384*m1**4*u2s**(-1) + 128*t2
     +     + 128*t2**2*u2s**(-1) + 160*u2 - 32*u2**2*s4**(-1) )
      MM_qg = MM_qg + ANG_fin(1,5,-1,-1)*Ar2*Nc**3*st**2*Pi**2*
     + alphas**2*hardfac * ( 64*s*(s+t2)**(-1)*m1**2*u2*s4**(-1) - 64*s
     +    *(s+t2)**(-1)*m1**2 + 32*s*(s+t2)**(-1)*m1**4*s4**(-1) + 32*s
     +    *(s+t2)**(-1)*u2**2*s4**(-1) + 64*s*m1**2*s4**(-1) - 256*s*
     +    m1**2*u2s**(-1) + 128*s*m1**4*s4**(-1)*u2s**(-1) - 128*s*t2*
     +    u2s**(-1) + 64*s*u2*s4**(-1) - 64*s + 64*(s+t2)**(-1)*m1**2*
     +    u2 - 64*(s+t2)**(-1)*m1**2*u2**2*s4**(-1) + 64*(s+t2)**(-1)*
     +    m1**4*u2*s4**(-1) - 32*(s+t2)**(-1)*m1**4 - 32*(s+t2)**(-1)*
     +    u2**2 - 64*m1**2*t2*s4**(-1) - 128*m1**2*t2*u2s**(-1) - 128*
     +    m1**2*u2*s4**(-1) - 160*m1**2 - 128*m1**4*t2*s4**(-1)*
     +    u2s**(-1) - 32*m1**4*s4**(-1) - 384*m1**4*u2s**(-1) - 128*t2
     +     - 128*t2**2*u2s**(-1) - 160*u2 + 32*u2**2*s4**(-1) )
      MM_qg = MM_qg + ANG_fin(1,7,-1,1)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 128*s**(-1)*m1**2*t2*s4**(-1)*u2s**(-1) + 128*
     +    s**(-1)*m1**2*s4**(-1) - 64*s**(-1)*m1**2*u2s**(-1) + 128*
     +    s**(-1)*m1**4*s4**(-1)*u2s**(-1) + 64*s**(-1)*t2*s4**(-1) + 
     +    32*s**(-1)*t2**2*s4**(-1)*u2s**(-1) + 64*s**(-1)*u2*s4**(-1)
     +     + 32*s*s4**(-1)*u2s**(-1) + 128*m1**2*s4**(-1)*u2s**(-1) + 
     +    64*t2*s4**(-1)*u2s**(-1) + 64*s4**(-1) )
      MM_qg = MM_qg + ANG_fin(1,7,-1,1)*Ar2*Nc**3*st**2*Pi**2*alphas**2
     + *hardfac * (  - 128*s**(-1)*m1**2*t2*s4**(-1)*u2s**(-1) - 128*
     +    s**(-1)*m1**2*s4**(-1) + 64*s**(-1)*m1**2*u2s**(-1) - 128*
     +    s**(-1)*m1**4*s4**(-1)*u2s**(-1) - 64*s**(-1)*t2*s4**(-1) - 
     +    32*s**(-1)*t2**2*s4**(-1)*u2s**(-1) - 64*s**(-1)*u2*s4**(-1)
     +     - 32*s*s4**(-1)*u2s**(-1) - 128*m1**2*s4**(-1)*u2s**(-1) - 
     +    64*t2*s4**(-1)*u2s**(-1) - 64*s4**(-1) )
      MM_qg = MM_qg + ANG_fin(2,0,-1,0)*Ar2*Nc**(-1)*st**2*Pi**2*
     + alphas**2*hardfac * (  - 64 + 32*s**(-1)*m1**2*u2*s4**(-1) - 32*
     +    s**(-1)*t2*u2*s4**(-1) + 32*s**(-1)*u2 - 32*s**(-1)*u2**2*
     +    s4**(-1) + 64*s*m1**2*s4**(-1)*u2s**(-1) + 32*s*t2*s4**(-1)*
     +    u2s**(-1) + 128*s*s4**(-1) - 64*s*u2s**(-1) + 64*s**2*
     +    s4**(-1)*u2s**(-1) + 32*m1**2*t2*s4**(-1)*u2s**(-1) - 64*
     +    m1**2*u2s**(-1) + 32*t2*s4**(-1) + 32*u2*s4**(-1) )
      MM_qg = MM_qg + ANG_fin(2,0,-1,0)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 64 - 32*s**(-1)*m1**2*u2*s4**(-1) + 32*s**(-1)*t2*u2
     +    *s4**(-1) - 32*s**(-1)*u2 + 32*s**(-1)*u2**2*s4**(-1) - 64*s*
     +    m1**2*s4**(-1)*u2s**(-1) - 32*s*t2*s4**(-1)*u2s**(-1) - 128*s
     +    *s4**(-1) + 64*s*u2s**(-1) - 64*s**2*s4**(-1)*u2s**(-1) - 32*
     +    m1**2*t2*s4**(-1)*u2s**(-1) + 64*m1**2*u2s**(-1) - 32*t2*
     +    s4**(-1) - 32*u2*s4**(-1) )
      MM_qg = MM_qg + ANG_fin(2,5,-1,-2)*Ar2*Nc**(-1)*st**2*Pi**2*
     + alphas**2*hardfac * ( 32*s*m1**2 )
      MM_qg = MM_qg + ANG_fin(2,5,-1,-2)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * (  - 64*s*m1**2 )
      MM_qg = MM_qg + ANG_fin(2,5,-1,-2)*Ar2*Nc**3*st**2*Pi**2*
     + alphas**2*hardfac * ( 32*s*m1**2 )
      MM_qg = MM_qg + ANG_fin(2,5,-1,-1)*Ar2*Nc**(-1)*st**2*Pi**2*
     + alphas**2*hardfac * (  - 32*s*m1**2*t2**(-1) + 64*s*m1**2*t2*
     +    s4**(-1)*u2s**(-1) - 64*s*m1**2*s4**(-1) + 32*s*m1**2*
     +    u2s**(-1) - 64*s*m1**4*s4**(-1)*u2s**(-1) + 64*s*t2*s4**(-1)
     +     - 32*s*t2*u2s**(-1) - 32*s**2*m1**2*s4**(-1)*u2s**(-1) + 32*
     +    s**2*t2*s4**(-1)*u2s**(-1) + 32*m1**2*t2**(-1)*u2 + 32*m1**2*
     +    t2*s4**(-1) - 32*m1**2*u2*s4**(-1) + 32*m1**2 + 32*m1**4*
     +    t2**(-1) + 64*m1**4*t2*s4**(-1)*u2s**(-1) - 32*m1**4*s4**(-1)
     +     - 64*m1**4*u2s**(-1) + 64*m1**6*t2**(-1)*u2s**(-1) - 64*
     +    m1**6*s4**(-1)*u2s**(-1) + 32*t2*u2*s4**(-1) - 32*t2 )
      MM_qg = MM_qg + ANG_fin(2,5,-1,-1)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 64*s*m1**2*t2**(-1) - 64*s*m1**2*t2*s4**(-1)*
     +    u2s**(-1) + 64*s*m1**2*s4**(-1) - 32*s*m1**2*u2s**(-1) + 64*s
     +    *m1**4*s4**(-1)*u2s**(-1) - 64*s*t2*s4**(-1) + 32*s*t2*
     +    u2s**(-1) + 32*s**2*m1**2*s4**(-1)*u2s**(-1) - 32*s**2*t2*
     +    s4**(-1)*u2s**(-1) - 64*m1**2*t2**(-1)*u2 - 32*m1**2*t2*
     +    s4**(-1) - 96*m1**2*t2*u2s**(-1) + 32*m1**2*u2*s4**(-1) - 32*
     +    m1**2 - 64*m1**4*t2**(-1) - 64*m1**4*t2*s4**(-1)*u2s**(-1) + 
     +    32*m1**4*s4**(-1) + 192*m1**4*u2s**(-1) - 128*m1**6*t2**(-1)*
     +    u2s**(-1) + 64*m1**6*s4**(-1)*u2s**(-1) - 32*t2*u2*s4**(-1)
     +     + 32*t2 + 32*t2**2*u2s**(-1) )
      MM_qg = MM_qg + ANG_fin(2,5,-1,-1)*Ar2*Nc**3*st**2*Pi**2*
     + alphas**2*hardfac * (  - 32*s*m1**2*t2**(-1) + 32*m1**2*t2**(-1)
     +    *u2 + 96*m1**2*t2*u2s**(-1) + 32*m1**4*t2**(-1) - 128*m1**4*
     +    u2s**(-1) + 64*m1**6*t2**(-1)*u2s**(-1) - 32*t2**2*u2s**(-1)
     +     )
      MM_qg = MM_qg + ANG_fin(2,6,-1,-2)*Ar2*Nc**(-1)*st**2*Pi**2*
     + alphas**2*hardfac * (  - 256*s*m1**2 + 128*s*t2 + 320*s*u2 - 128
     +    *s*s4 + 64*s**2 - 256*m1**2*t2 - 512*m1**2*u2 + 256*m1**2*s4
     +     + 128*m1**4 - 128*m1**6*u2s**(-1) + 320*t2*u2 - 128*t2*s4 + 
     +    64*t2**2 - 320*u2*s4 + 256*u2**2 + 64*s4**2 )
      MM_qg = MM_qg + ANG_fin(2,6,-1,-2)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 512*s*m1**2 - 256*s*t2 - 640*s*u2 + 256*s*s4 - 128*
     +    s**2 + 512*m1**2*t2 + 1024*m1**2*u2 - 512*m1**2*s4 - 256*
     +    m1**4 + 256*m1**6*u2s**(-1) - 640*t2*u2 + 256*t2*s4 - 128*
     +    t2**2 + 640*u2*s4 - 512*u2**2 - 128*s4**2 )
      MM_qg = MM_qg + ANG_fin(2,6,-1,-2)*Ar2*Nc**3*st**2*Pi**2*
     + alphas**2*hardfac * (  - 256*s*m1**2 + 128*s*t2 + 320*s*u2 - 128
     +    *s*s4 + 64*s**2 - 256*m1**2*t2 - 512*m1**2*u2 + 256*m1**2*s4
     +     + 128*m1**4 - 128*m1**6*u2s**(-1) + 320*t2*u2 - 128*t2*s4 + 
     +    64*t2**2 - 320*u2*s4 + 256*u2**2 + 64*s4**2 )
      MM_qg = MM_qg + ANG_fin(2,6,-1,-1)*Ar2*Nc**(-1)*st**2*Pi**2*
     + alphas**2*hardfac * (  - 64*s**(-1)*m1**2*t2*u2*s4**(-1) + 32*
     +    s**(-1)*m1**2*u2 - 128*s**(-1)*m1**2*u2**2*s4**(-1) - 32*
     +    s**(-1)*t2*u2 + 128*s**(-1)*t2*u2**2*s4**(-1) + 32*s**(-1)*
     +    t2**2*u2*s4**(-1) - 96*s**(-1)*u2**2 + 64*s**(-1)*u2**3*
     +    s4**(-1) - 32*s*m1**2*t2**(-1) - 64*s*m1**2*t2*s4**(-1)*
     +    u2s**(-1) + 128*s*m1**2*s4**(-1) - 32*s*m1**2*u2s**(-1) + 128
     +    *s*m1**4*s4**(-1)*u2s**(-1) - 64*s*t2*s4**(-1) + 32*s*t2*
     +    u2s**(-1) - 32*s*t2**2*s4**(-1)*u2s**(-1) + 32*s*u2*s4**(-1)
     +     - 96*s + 32*s**2*m1**2*s4**(-1)*u2s**(-1) - 64*s**2*t2*
     +    s4**(-1)*u2s**(-1) - 64*s**2*s4**(-1) + 32*s**2*u2s**(-1) - 
     +    32*s**3*s4**(-1)*u2s**(-1) + 32*m1**2*t2**(-1)*u2 + 96*m1**2*
     +    t2*u2s**(-1) - 96*m1**2*t2**2*s4**(-1)*u2s**(-1) - 64*m1**2*
     +    u2*s4**(-1) + 32*m1**2 + 32*m1**4*t2**(-1) + 256*m1**4*t2*
     +    s4**(-1)*u2s**(-1) - 320*m1**4*u2s**(-1) + 64*m1**6*t2**(-1)*
     +    u2s**(-1) )
      MM_qg = MM_qg + ANG_fin(2,6,-1,-1)*Ar2*Nc**(-1)*st**2*Pi**2*
     + alphas**2*hardfac * (  - 128*m1**6*s4**(-1)*u2s**(-1) + 128*t2*
     +    u2*s4**(-1) - 128*t2 - 224*u2 + 128*u2**2*s4**(-1) + 128*s4 )
      MM_qg = MM_qg + ANG_fin(2,6,-1,-1)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 64*s**(-1)*m1**2*t2*u2*s4**(-1) - 32*s**(-1)*m1**2*
     +    u2 + 128*s**(-1)*m1**2*u2**2*s4**(-1) + 32*s**(-1)*t2*u2 - 
     +    128*s**(-1)*t2*u2**2*s4**(-1) - 32*s**(-1)*t2**2*u2*s4**(-1)
     +     + 96*s**(-1)*u2**2 - 64*s**(-1)*u2**3*s4**(-1) + 32*s*
     +    (s+t2)**(-1)*m1**2 + 128*s*(s+t2)**(-1)*m1**4*s4**(-1) + 32*s
     +    *(s+t2)**(-1)*u2 + 64*s*m1**2*t2**(-1) + 128*s*m1**2*t2*
     +    s4**(-1)*u2s**(-1) - 32*s*m1**2*s4**(-1) - 64*s*m1**2*
     +    u2s**(-1) - 32*s*m1**4*s4**(-1)*u2s**(-1) + 32*s*t2*s4**(-1)
     +     - 64*s*t2*u2s**(-1) + 64*s*t2**2*s4**(-1)*u2s**(-1) + 32*s*
     +    u2*s4**(-1) + 192*s + 64*s**2*m1**2*s4**(-1)*u2s**(-1) + 128*
     +    s**2*t2*s4**(-1)*u2s**(-1) + 64*s**2*s4**(-1) - 64*s**2*
     +    u2s**(-1) + 64*s**3*s4**(-1)*u2s**(-1) - 160*(s+t2)**(-1)*
     +    m1**2*u2 + 192*(s+t2)**(-1)*m1**2*u2**2*s4**(-1) - 128*
     +    (s+t2)**(-1)*m1**4*u2*s4**(-1) + 32*(s+t2)**(-1)*m1**4 - 32*
     +    (s+t2)**(-1)*u2*u2s )
      MM_qg = MM_qg + ANG_fin(2,6,-1,-1)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 128*(s+t2)**(-1)*u2**2 - 64*(s+t2)**(-1)*u2**3*
     +    s4**(-1) - 64*m1**2*t2**(-1)*u2 - 32*m1**2*t2*s4**(-1) - 64*
     +    m1**2*t2*u2s**(-1) + 64*m1**2*t2**2*s4**(-1)*u2s**(-1) + 64*
     +    m1**2*u2*s4**(-1) - 64*m1**2 - 64*m1**4*t2**(-1) - 288*m1**4*
     +    t2*s4**(-1)*u2s**(-1) + 416*m1**4*u2s**(-1) - 128*m1**6*
     +    t2**(-1)*u2s**(-1) + 128*m1**6*s4**(-1)*u2s**(-1) - 64*t2*u2*
     +    s4**(-1) + 224*t2 - 32*t2**2*s4**(-1) + 192*u2 - 96*u2**2*
     +    s4**(-1) - 192*s4 )
      MM_qg = MM_qg + ANG_fin(2,6,-1,-1)*Ar2*Nc**3*st**2*Pi**2*
     + alphas**2*hardfac * (  - 32*s*(s+t2)**(-1)*m1**2 - 128*s*
     +    (s+t2)**(-1)*m1**4*s4**(-1) - 32*s*(s+t2)**(-1)*u2 - 32*s*
     +    m1**2*t2**(-1) - 64*s*m1**2*t2*s4**(-1)*u2s**(-1) - 96*s*
     +    m1**2*s4**(-1) + 96*s*m1**2*u2s**(-1) - 96*s*m1**4*s4**(-1)*
     +    u2s**(-1) + 32*s*t2*s4**(-1) + 32*s*t2*u2s**(-1) - 32*s*t2**2
     +    *s4**(-1)*u2s**(-1) - 64*s*u2*s4**(-1) - 96*s - 96*s**2*m1**2
     +    *s4**(-1)*u2s**(-1) - 64*s**2*t2*s4**(-1)*u2s**(-1) + 32*s**2
     +    *u2s**(-1) - 32*s**3*s4**(-1)*u2s**(-1) + 160*(s+t2)**(-1)*
     +    m1**2*u2 - 192*(s+t2)**(-1)*m1**2*u2**2*s4**(-1) + 128*
     +    (s+t2)**(-1)*m1**4*u2*s4**(-1) - 32*(s+t2)**(-1)*m1**4 + 32*
     +    (s+t2)**(-1)*u2*u2s - 128*(s+t2)**(-1)*u2**2 + 64*
     +    (s+t2)**(-1)*u2**3*s4**(-1) + 32*m1**2*t2**(-1)*u2 + 32*m1**2
     +    *t2*s4**(-1) - 32*m1**2*t2*u2s**(-1) + 32*m1**2*t2**2*
     +    s4**(-1)*u2s**(-1) + 32*m1**2 + 32*m1**4*t2**(-1) + 32*m1**4*
     +    t2*s4**(-1)*u2s**(-1) )
      MM_qg = MM_qg + ANG_fin(2,6,-1,-1)*Ar2*Nc**3*st**2*Pi**2*
     + alphas**2*hardfac * (  - 96*m1**4*u2s**(-1) + 64*m1**6*t2**(-1)*
     +    u2s**(-1) - 64*t2*u2*s4**(-1) - 96*t2 + 32*t2**2*s4**(-1) + 
     +    32*u2 - 32*u2**2*s4**(-1) + 64*s4 )
      MM_qg = MM_qg + ANG_fin(5,0,-2,0)*Ar2*Nc**(-1)*st**2*Pi**2*
     + alphas**2*hardfac * (  - 32*s**(-1)*m1**2*u2 - 32*m1**2 )
      MM_qg = MM_qg + ANG_fin(5,0,-2,0)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 64*s**(-1)*m1**2*u2 + 128*m1**2 )
      MM_qg = MM_qg + ANG_fin(5,0,-2,0)*Ar2*Nc**3*st**2*Pi**2*alphas**2
     + *hardfac * (  - 32*s**(-1)*m1**2*u2 - 96*m1**2 )
      MM_qg = MM_qg + ANG_fin(5,0,-1,0)*Ar2*Nc**(-1)*st**2*Pi**2*
     + alphas**2*hardfac * (  - 32 - 32*s**(-1)*m1**2*t2*s4**(-1) - 96*
     +    s**(-1)*m1**2*t2*u2s**(-1) - 96*s**(-1)*m1**2*u2*s4**(-1) - 
     +    32*s**(-1)*m1**2 - 64*s**(-1)*m1**4*t2*s4**(-1)*u2s**(-1) - 
     +    64*s**(-1)*m1**4*s4**(-1) - 64*s**(-1)*m1**4*u2s**(-1) - 64*
     +    s**(-1)*m1**6*s4**(-1)*u2s**(-1) - 64*s**(-1)*t2 - 32*s**(-1)
     +    *t2**2*u2s**(-1) - 32*s**(-1)*u2 - 32*s*m1**2*s4**(-1)*
     +    u2s**(-1) + 32*s*t2*s4**(-1)*u2s**(-1) + 32*m1**2*t2*s4**(-1)
     +    *u2s**(-1) - 128*m1**2*s4**(-1) - 64*m1**2*u2s**(-1) - 96*
     +    m1**4*s4**(-1)*u2s**(-1) + 32*t2*s4**(-1) - 64*t2*u2s**(-1) )
      MM_qg = MM_qg + ANG_fin(5,0,-1,0)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 32 + 64*s**(-1)*m1**2*t2*s4**(-1) + 192*s**(-1)*
     +    m1**2*t2*u2s**(-1) + 192*s**(-1)*m1**2*u2*s4**(-1) + 64*
     +    s**(-1)*m1**2 + 128*s**(-1)*m1**4*t2*s4**(-1)*u2s**(-1) + 128
     +    *s**(-1)*m1**4*s4**(-1) + 128*s**(-1)*m1**4*u2s**(-1) + 128*
     +    s**(-1)*m1**6*s4**(-1)*u2s**(-1) + 128*s**(-1)*t2 + 64*
     +    s**(-1)*t2**2*u2s**(-1) + 64*s**(-1)*u2 + 96*s*m1**2*s4**(-1)
     +    *u2s**(-1) - 32*s*t2*s4**(-1)*u2s**(-1) + 32*s*s4**(-1) - 64*
     +    m1**2*t2*s4**(-1)*u2s**(-1) + 256*m1**2*s4**(-1) + 160*m1**2*
     +    u2s**(-1) + 192*m1**4*s4**(-1)*u2s**(-1) - 32*t2*s4**(-1) + 
     +    160*t2*u2s**(-1) + 32*u2*s4**(-1) )
      MM_qg = MM_qg + ANG_fin(5,0,-1,0)*Ar2*Nc**3*st**2*Pi**2*alphas**2
     + *hardfac * (  - 32*s**(-1)*m1**2*t2*s4**(-1) - 96*s**(-1)*m1**2*
     +    t2*u2s**(-1) - 96*s**(-1)*m1**2*u2*s4**(-1) - 32*s**(-1)*
     +    m1**2 - 64*s**(-1)*m1**4*t2*s4**(-1)*u2s**(-1) - 64*s**(-1)*
     +    m1**4*s4**(-1) - 64*s**(-1)*m1**4*u2s**(-1) - 64*s**(-1)*
     +    m1**6*s4**(-1)*u2s**(-1) - 64*s**(-1)*t2 - 32*s**(-1)*t2**2*
     +    u2s**(-1) - 32*s**(-1)*u2 - 64*s*m1**2*s4**(-1)*u2s**(-1) - 
     +    32*s*s4**(-1) + 32*m1**2*t2*s4**(-1)*u2s**(-1) - 128*m1**2*
     +    s4**(-1) - 96*m1**2*u2s**(-1) - 96*m1**4*s4**(-1)*u2s**(-1)
     +     - 96*t2*u2s**(-1) - 32*u2*s4**(-1) )
      MM_qg = MM_qg + ANG_fin(5,6,-1,-1)*Ar2*Nc**(-1)*st**2*Pi**2*
     + alphas**2*hardfac * (  - 96*s**(-1)*m1**2*t2 - 96*s**(-1)*m1**2*
     +    t2**2*u2s**(-1) - 32*s**(-1)*m1**2*u2 - 128*s**(-1)*m1**4*t2*
     +    u2s**(-1) - 64*s**(-1)*m1**4 - 64*s**(-1)*m1**6*u2s**(-1) - 
     +    96*s**(-1)*t2*u2 - 96*s**(-1)*t2**2 - 32*s**(-1)*t2**3*
     +    u2s**(-1) - 32*s**(-1)*u2**2 + 32*s*m1**2*t2**(-1) + 32*s*
     +    m1**2*u2s**(-1) - 32*s*t2*u2s**(-1) - 32*s - 32*m1**2*
     +    t2**(-1)*u2 - 64*m1**2*t2*u2s**(-1) - 32*m1**2 - 32*m1**4*
     +    t2**(-1) - 64*m1**6*t2**(-1)*u2s**(-1) - 128*t2 - 64*t2**2*
     +    u2s**(-1) - 64*u2 )
      MM_qg = MM_qg + ANG_fin(5,6,-1,-1)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 96*s**(-1)*m1**2*t2 + 96*s**(-1)*m1**2*t2**2*
     +    u2s**(-1) + 32*s**(-1)*m1**2*u2 + 128*s**(-1)*m1**4*t2*
     +    u2s**(-1) + 64*s**(-1)*m1**4 + 64*s**(-1)*m1**6*u2s**(-1) + 
     +    96*s**(-1)*t2*u2 + 96*s**(-1)*t2**2 + 32*s**(-1)*t2**3*
     +    u2s**(-1) + 32*s**(-1)*u2**2 - 64*s*(s+t2)**(-1)*m1**2*u2*
     +    s4**(-1) + 64*s*(s+t2)**(-1)*m1**2 - 32*s*(s+t2)**(-1)*m1**4*
     +    s4**(-1) - 32*s*(s+t2)**(-1)*u2**2*s4**(-1) - 64*s*m1**2*
     +    t2**(-1) - 96*s*m1**2*s4**(-1) - 64*s*m1**2*u2s**(-1) - 128*s
     +    *m1**4*s4**(-1)*u2s**(-1) + 64*s*t2*u2s**(-1) - 32*s*u2*
     +    s4**(-1) + 64*s + 32*s**2*t2*s4**(-1)*u2s**(-1) + 32*s**2*
     +    s4**(-1) - 32*s**2*u2s**(-1) + 32*s**3*s4**(-1)*u2s**(-1) - 
     +    64*(s+t2)**(-1)*m1**2*u2 + 64*(s+t2)**(-1)*m1**2*u2**2*
     +    s4**(-1) - 64*(s+t2)**(-1)*m1**4*u2*s4**(-1) + 32*
     +    (s+t2)**(-1)*m1**4 + 32*(s+t2)**(-1)*u2**2 + 64*m1**2*
     +    t2**(-1)*u2 )
      MM_qg = MM_qg + ANG_fin(5,6,-1,-1)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 160*m1**2*t2*u2s**(-1) + 64*m1**2*u2*s4**(-1) + 128*
     +    m1**2 + 64*m1**4*t2**(-1) + 128*m1**4*u2s**(-1) + 128*m1**6*
     +    t2**(-1)*u2s**(-1) + 224*t2 + 96*t2**2*u2s**(-1) + 160*u2 )
      MM_qg = MM_qg + ANG_fin(5,6,-1,-1)*Ar2*Nc**3*st**2*Pi**2*
     + alphas**2*hardfac * ( 64*s*(s+t2)**(-1)*m1**2*u2*s4**(-1) - 64*s
     +    *(s+t2)**(-1)*m1**2 + 32*s*(s+t2)**(-1)*m1**4*s4**(-1) + 32*s
     +    *(s+t2)**(-1)*u2**2*s4**(-1) + 32*s*m1**2*t2**(-1) + 96*s*
     +    m1**2*s4**(-1) + 32*s*m1**2*u2s**(-1) + 128*s*m1**4*s4**(-1)*
     +    u2s**(-1) - 32*s*t2*u2s**(-1) + 32*s*u2*s4**(-1) - 32*s - 32*
     +    s**2*t2*s4**(-1)*u2s**(-1) - 32*s**2*s4**(-1) + 32*s**2*
     +    u2s**(-1) - 32*s**3*s4**(-1)*u2s**(-1) + 64*(s+t2)**(-1)*
     +    m1**2*u2 - 64*(s+t2)**(-1)*m1**2*u2**2*s4**(-1) + 64*
     +    (s+t2)**(-1)*m1**4*u2*s4**(-1) - 32*(s+t2)**(-1)*m1**4 - 32*
     +    (s+t2)**(-1)*u2**2 - 32*m1**2*t2**(-1)*u2 - 96*m1**2*t2*
     +    u2s**(-1) - 64*m1**2*u2*s4**(-1) - 96*m1**2 - 32*m1**4*
     +    t2**(-1) - 128*m1**4*u2s**(-1) - 64*m1**6*t2**(-1)*u2s**(-1)
     +     - 96*t2 - 32*t2**2*u2s**(-1) - 96*u2 )
      MM_qg = MM_qg + ANG_fin(5,7,-2,1)*Ar2*Nc**(-1)*st**2*Pi**2*
     + alphas**2*hardfac * (  - 32*s**(-1)*m1**2 )
      MM_qg = MM_qg + ANG_fin(5,7,-2,1)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 64*s**(-1)*m1**2 )
      MM_qg = MM_qg + ANG_fin(5,7,-2,1)*Ar2*Nc**3*st**2*Pi**2*alphas**2
     + *hardfac * (  - 32*s**(-1)*m1**2 )
      MM_qg = MM_qg + ANG_fin(5,7,-1,1)*Ar2*Nc**(-1)*st**2*Pi**2*
     + alphas**2*hardfac * (  - 32*s**(-1)*t2*s4**(-1) - 32*s**(-1)*u2*
     +    s4**(-1) + 32*s**(-1) - 32*m1**2*s4**(-1)*u2s**(-1) - 32*t2*
     +    s4**(-1)*u2s**(-1) - 32*s4**(-1) )
      MM_qg = MM_qg + ANG_fin(5,7,-1,1)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 64*s**(-1)*t2*s4**(-1) + 64*s**(-1)*u2*s4**(-1) - 64
     +    *s**(-1) + 64*m1**2*s4**(-1)*u2s**(-1) + 64*t2*s4**(-1)*
     +    u2s**(-1) + 64*s4**(-1) )
      MM_qg = MM_qg + ANG_fin(5,7,-1,1)*Ar2*Nc**3*st**2*Pi**2*alphas**2
     + *hardfac * (  - 32*s**(-1)*t2*s4**(-1) - 32*s**(-1)*u2*s4**(-1)
     +     + 32*s**(-1) - 32*m1**2*s4**(-1)*u2s**(-1) - 32*t2*s4**(-1)*
     +    u2s**(-1) - 32*s4**(-1) )
      MM_qg = MM_qg + ANG_fin(6,0,-2,0)*Ar2*Nc**(-1)*st**2*Pi**2*
     + alphas**2*hardfac * ( 64*s - 128*m1**2 - 128*m1**4*u2s**(-1) - 
     +    128*m1**6*u2s**(-2) + 64*t2 + 64*u2 - 64*s4 )
      MM_qg = MM_qg + ANG_fin(6,0,-2,0)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * (  - 128*s + 256*m1**2 + 256*m1**4*u2s**(-1) + 256*
     +    m1**6*u2s**(-2) - 128*t2 - 128*u2 + 128*s4 )
      MM_qg = MM_qg + ANG_fin(6,0,-2,0)*Ar2*Nc**3*st**2*Pi**2*alphas**2
     + *hardfac * ( 64*s - 128*m1**2 - 128*m1**4*u2s**(-1) - 128*m1**6*
     +    u2s**(-2) + 64*t2 + 64*u2 - 64*s4 )
      MM_qg = MM_qg + ANG_fin(6,0,-1,0)*Ar2*Nc**(-1)*st**2*Pi**2*
     + alphas**2*hardfac * (  - 32 - 416*s**(-1)*m1**2*t2*s4**(-1) + 
     +    192*s**(-1)*m1**2*t2*u2s**(-1) - 256*s**(-1)*m1**2*t2**2*
     +    s4**(-1)*u2s**(-1) - 192*s**(-1)*m1**2*u2*s4**(-1) + 384*
     +    s**(-1)*m1**2 - 256*s**(-1)*m1**4*t2*s4**(-1)*u2s**(-1) + 192
     +    *s**(-1)*m1**4*u2s**(-1) - 224*s**(-1)*t2*u2*s4**(-1) - 64*
     +    s**(-1)*t2 - 192*s**(-1)*t2**2*s4**(-1) + 32*s**(-1)*t2**2*
     +    u2s**(-1) - 64*s**(-1)*t2**3*s4**(-1)*u2s**(-1) - 64*s**(-1)*
     +    u2 - 128*s**(-1)*u2**2*s4**(-1) + 128*s**(-1)*s4 - 256*s*
     +    m1**2*s4**(-1)*u2s**(-1) - 128*s*m1**4*s4**(-1)*u2s**(-2) - 
     +    160*s*t2*s4**(-1)*u2s**(-1) - 192*s*s4**(-1) + 64*s*u2s**(-1)
     +     - 64*s**2*s4**(-1)*u2s**(-1) - 512*m1**2*t2*s4**(-1)*
     +    u2s**(-1) - 512*m1**2*s4**(-1) + 256*m1**2*u2s**(-1) - 128*
     +    m1**4*t2*s4**(-1)*u2s**(-2) - 512*m1**4*s4**(-1)*u2s**(-1) - 
     +    256*m1**6*s4**(-1)*u2s**(-2) - 352*t2*s4**(-1) + 64*t2*
     +    u2s**(-1) )
      MM_qg = MM_qg + ANG_fin(6,0,-1,0)*Ar2*Nc**(-1)*st**2*Pi**2*
     + alphas**2*hardfac * (  - 160*t2**2*s4**(-1)*u2s**(-1) - 256*u2*
     +    s4**(-1) )
      MM_qg = MM_qg + ANG_fin(6,0,-1,0)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * ( 64 + 416*s**(-1)*m1**2*t2*s4**(-1) - 192*s**(-1)*
     +    m1**2*t2*u2s**(-1) + 256*s**(-1)*m1**2*t2**2*s4**(-1)*
     +    u2s**(-1) + 192*s**(-1)*m1**2*u2*s4**(-1) - 384*s**(-1)*m1**2
     +     + 256*s**(-1)*m1**4*t2*s4**(-1)*u2s**(-1) - 192*s**(-1)*
     +    m1**4*u2s**(-1) + 224*s**(-1)*t2*u2*s4**(-1) + 64*s**(-1)*t2
     +     + 192*s**(-1)*t2**2*s4**(-1) - 32*s**(-1)*t2**2*u2s**(-1) + 
     +    64*s**(-1)*t2**3*s4**(-1)*u2s**(-1) + 64*s**(-1)*u2 + 128*
     +    s**(-1)*u2**2*s4**(-1) - 128*s**(-1)*s4 - 32*s*(s+t2)**(-1)*
     +    m1**2*s4**(-1) + 32*s*(s+t2)**(-1)*u2*s4**(-1) - 32*s*
     +    (s+t2)**(-1) + 288*s*m1**2*s4**(-1)*u2s**(-1) + 128*s*m1**4*
     +    s4**(-1)*u2s**(-2) + 128*s*t2*s4**(-1)*u2s**(-1) + 224*s*
     +    s4**(-1) + 32*s**2*s4**(-1)*u2s**(-1) + 192*(s+t2)**(-1)*
     +    m1**2*u2*s4**(-1) - 128*(s+t2)**(-1)*m1**2*u2**2*s4**(-2) - 
     +    64*(s+t2)**(-1)*m1**2 + 32*(s+t2)**(-1)*m1**4*u2*s4**(-2) + 
     +    96*(s+t2)**(-1)*u2 )
      MM_qg = MM_qg + ANG_fin(6,0,-1,0)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * (  - 128*(s+t2)**(-1)*u2**2*s4**(-1) + 96*(s+t2)**(-1)
     +    *u2**3*s4**(-2) + 544*m1**2*t2*s4**(-1)*u2s**(-1) - 32*m1**2*
     +    u2*s4**(-2) + 640*m1**2*s4**(-1) - 160*m1**2*u2s**(-1) + 128*
     +    m1**4*t2*s4**(-1)*u2s**(-2) + 640*m1**4*s4**(-1)*u2s**(-1) + 
     +    128*m1**4*u2s**(-2) + 256*m1**6*s4**(-1)*u2s**(-2) + 384*t2*
     +    s4**(-1) - 32*t2*u2s**(-1) + 160*t2**2*s4**(-1)*u2s**(-1) + 
     +    256*u2*s4**(-1) + 96*u2**2*s4**(-2) )
      MM_qg = MM_qg + ANG_fin(6,0,-1,0)*Ar2*Nc**3*st**2*Pi**2*alphas**2
     + *hardfac * (  - 32 + 32*s*(s+t2)**(-1)*m1**2*s4**(-1) - 32*s*
     +    (s+t2)**(-1)*u2*s4**(-1) + 32*s*(s+t2)**(-1) - 32*s*m1**2*
     +    s4**(-1)*u2s**(-1) + 32*s*t2*s4**(-1)*u2s**(-1) - 32*s*
     +    s4**(-1) - 64*s*u2s**(-1) + 32*s**2*s4**(-1)*u2s**(-1) - 192*
     +    (s+t2)**(-1)*m1**2*u2*s4**(-1) + 128*(s+t2)**(-1)*m1**2*u2**2
     +    *s4**(-2) + 64*(s+t2)**(-1)*m1**2 - 32*(s+t2)**(-1)*m1**4*u2*
     +    s4**(-2) - 96*(s+t2)**(-1)*u2 + 128*(s+t2)**(-1)*u2**2*
     +    s4**(-1) - 96*(s+t2)**(-1)*u2**3*s4**(-2) - 32*m1**2*t2*
     +    s4**(-1)*u2s**(-1) + 32*m1**2*u2*s4**(-2) - 128*m1**2*
     +    s4**(-1) - 96*m1**2*u2s**(-1) - 128*m1**4*s4**(-1)*u2s**(-1)
     +     - 128*m1**4*u2s**(-2) - 32*t2*s4**(-1) - 32*t2*u2s**(-1) - 
     +    96*u2**2*s4**(-2) )
      MM_qg = MM_qg + ANG_fin(6,7,-1,1)*Ar2*Nc**(-1)*st**2*Pi**2*
     + alphas**2*hardfac * ( 128*s**(-1)*m1**2*t2*s4**(-1)*u2s**(-1) + 
     +    128*s**(-1)*m1**2*s4**(-1) - 64*s**(-1)*m1**2*u2s**(-1) + 128
     +    *s**(-1)*m1**4*s4**(-1)*u2s**(-1) + 64*s**(-1)*t2*s4**(-1) + 
     +    32*s**(-1)*t2**2*s4**(-1)*u2s**(-1) + 64*s**(-1)*u2*s4**(-1)
     +     + 32*s*s4**(-1)*u2s**(-1) + 128*m1**2*s4**(-1)*u2s**(-1) + 
     +    64*t2*s4**(-1)*u2s**(-1) + 64*s4**(-1) )
      MM_qg = MM_qg + ANG_fin(6,7,-1,1)*Ar2*Nc*st**2*Pi**2*alphas**2*
     + hardfac * (  - 128*s**(-1)*m1**2*t2*s4**(-1)*u2s**(-1) - 128*
     +    s**(-1)*m1**2*s4**(-1) + 64*s**(-1)*m1**2*u2s**(-1) - 128*
     +    s**(-1)*m1**4*s4**(-1)*u2s**(-1) - 64*s**(-1)*t2*s4**(-1) - 
     +    32*s**(-1)*t2**2*s4**(-1)*u2s**(-1) - 64*s**(-1)*u2*s4**(-1)
     +     - 32*s*s4**(-1)*u2s**(-1) - 128*m1**2*s4**(-1)*u2s**(-1) - 
     +    64*t2*s4**(-1)*u2s**(-1) - 64*s4**(-1) )


c               the prefactor for the scaling functions 
      LE_QG = MM_qg * m1**2/4.D0
      
c               the phase space except for 1/s**2 
      LE_QG = LE_QG / ( 16.D0 * pi**2 )**2 / 2.D0 * s4/(s4+m1**2)
      
c               in this case multiply luminosity by hand
      LE_QG = LE_QG * lumi(1)
      
c               the averaging factors
      LE_QG = LE_QG /4.D0 /Nc /(Nc**2-1.D0)

      return
      end

